@@ -1,7 +1,7 @@
 #[macro_use]
 extern crate bencher;
 
-use bencher::Bencher;
+use bencher::{black_box, Bencher};
 use borsh::{BorshDeserialize, BorshSerialize};
 use near_primitives::time::Clock;
 
@@ -9,11 +9,13 @@ use near_crypto::{KeyType, PublicKey, Signature};
 use near_primitives::account::Account;
 use near_primitives::block::{genesis_chunks, Block};
 use near_primitives::hash::CryptoHash;
+use near_primitives::merkle::combine_hash;
 use near_primitives::test_utils::account_new;
 use near_primitives::transaction::{Action, SignedTransaction, Transaction, TransferAction};
-use near_primitives::types::{AccountId, EpochId, StateRoot};
+use near_primitives::types::{EpochId, StateRoot};
 use near_primitives::validator_signer::InMemoryValidatorSigner;
 use near_primitives::version::PROTOCOL_VERSION;
+use near_primitives_core::types::MerkleHash;
 use num_rational::Rational;
 
 fn create_transaction() -> SignedTransaction {
@@ -45,17 +47,16 @@ fn create_block() -> Block {
         1_000,
         CryptoHash::default(),
     );
-    let signer = InMemoryValidatorSigner::from_random(AccountId::test_account(), KeyType::ED25519);
+    let signer = InMemoryValidatorSigner::from_random("test".parse().unwrap(), KeyType::ED25519);
     Block::produce(
+        PROTOCOL_VERSION,
         PROTOCOL_VERSION,
         genesis.header(),
         10,
-        #[cfg(feature = "protocol_feature_block_header_v3")]
-        (genesis.header().block_ordinal() + 1),
+        genesis.header().block_ordinal() + 1,
         vec![genesis.chunks()[0].clone()],
         EpochId::default(),
         EpochId::default(),
-        #[cfg(feature = "protocol_feature_block_header_v3")]
         None,
         vec![],
         Rational::from_integer(0),
@@ -78,7 +79,7 @@ fn serialize_tx(bench: &mut Bencher) {
     let t = create_transaction();
     bench.iter(|| {
         let bytes = t.try_to_vec().unwrap();
-        assert!(bytes.len() > 0);
+        assert!(!bytes.is_empty());
     });
 }
 
@@ -95,7 +96,7 @@ fn serialize_block(bench: &mut Bencher) {
     let b = create_block();
     bench.iter(|| {
         let bytes = b.try_to_vec().unwrap();
-        assert!(bytes.len() > 0);
+        assert!(!bytes.is_empty());
     });
 }
 
@@ -112,7 +113,7 @@ fn serialize_account(bench: &mut Bencher) {
     let acc = create_account();
     bench.iter(|| {
         let bytes = acc.try_to_vec().unwrap();
-        assert!(bytes.len() > 0);
+        assert!(!bytes.is_empty());
     });
 }
 
@@ -125,6 +126,15 @@ fn deserialize_account(bench: &mut Bencher) {
     });
 }
 
+fn combine_hash_bench(bench: &mut Bencher) {
+    let a = MerkleHash::default();
+    let b = MerkleHash::default();
+    bench.iter(|| {
+        let res = combine_hash(black_box(&a), black_box(&b));
+        black_box(res)
+    });
+}
+
 benchmark_group!(
     benches,
     serialize_tx,
@@ -132,6 +142,7 @@ benchmark_group!(
     serialize_block,
     deserialize_block,
     serialize_account,
-    deserialize_account
+    deserialize_account,
+    combine_hash_bench,
 );
 benchmark_main!(benches);
