@@ -2,6 +2,7 @@ use std::cmp::max;
 
 use crate::time::{Clock, Utc};
 use borsh::{BorshDeserialize, BorshSerialize};
+
 use chrono::DateTime;
 use near_crypto::Signature;
 use num_rational::Rational;
@@ -19,14 +20,13 @@ use crate::sharding::{
     ChunkHashHeight, EncodedShardChunk, ReedSolomonWrapper, ShardChunk, ShardChunkHeader,
     ShardChunkHeaderV1,
 };
-#[cfg(feature = "protocol_feature_block_header_v3")]
-use crate::types::NumBlocks;
-use crate::types::{Balance, BlockHeight, EpochId, Gas, NumShards, StateRoot};
+use crate::types::{Balance, BlockHeight, EpochId, Gas, NumBlocks, NumShards, StateRoot};
 use crate::utils::to_timestamp;
 use crate::validator_signer::{EmptyValidatorSigner, ValidatorSigner};
 use crate::version::{ProtocolVersion, SHARD_CHUNK_HEADER_UPGRADE_VERSION};
 use std::ops::Index;
 
+#[cfg_attr(feature = "deepsize_feature", derive(deepsize::DeepSizeOf))]
 #[derive(BorshSerialize, BorshDeserialize, Clone, Debug, Eq, PartialEq, Default)]
 pub struct GenesisId {
     /// Chain Id
@@ -35,6 +35,7 @@ pub struct GenesisId {
     pub hash: CryptoHash,
 }
 
+#[cfg_attr(feature = "deepsize_feature", derive(deepsize::DeepSizeOf))]
 #[derive(BorshSerialize, BorshDeserialize, Clone, Debug, Eq, PartialEq)]
 pub enum BlockValidityError {
     InvalidStateRoot,
@@ -45,6 +46,7 @@ pub enum BlockValidityError {
     InvalidChallengeRoot,
 }
 
+#[cfg_attr(feature = "deepsize_feature", derive(deepsize::DeepSizeOf))]
 #[derive(BorshSerialize, BorshDeserialize, Debug, Clone, Eq, PartialEq)]
 pub struct BlockV1 {
     pub header: BlockHeader,
@@ -56,6 +58,7 @@ pub struct BlockV1 {
     pub vrf_proof: near_crypto::vrf::Proof,
 }
 
+#[cfg_attr(feature = "deepsize_feature", derive(deepsize::DeepSizeOf))]
 #[derive(BorshSerialize, BorshDeserialize, Debug, Clone, Eq, PartialEq)]
 pub struct BlockV2 {
     pub header: BlockHeader,
@@ -69,6 +72,7 @@ pub struct BlockV2 {
 
 /// Versioned Block data structure.
 /// For each next version, document what are the changes between versions.
+#[cfg_attr(feature = "deepsize_feature", derive(deepsize::DeepSizeOf))]
 #[derive(BorshSerialize, BorshDeserialize, Debug, Clone, Eq, PartialEq)]
 pub enum Block {
     BlockV1(Box<BlockV1>),
@@ -89,7 +93,7 @@ pub fn genesis_chunks(
         .map(|i| {
             let (encoded_chunk, _) = EncodedShardChunk::new(
                 CryptoHash::default(),
-                state_roots[i as usize % state_roots.len()].clone(),
+                state_roots[i as usize % state_roots.len()],
                 CryptoHash::default(),
                 genesis_height,
                 i,
@@ -100,7 +104,7 @@ pub fn genesis_chunks(
                 CryptoHash::default(),
                 vec![],
                 vec![],
-                &vec![],
+                &[],
                 CryptoHash::default(),
                 &EmptyValidatorSigner::default(),
                 genesis_protocol_version,
@@ -115,14 +119,14 @@ pub fn genesis_chunks(
 
 impl Block {
     fn block_from_protocol_version(
-        protocol_version: ProtocolVersion,
+        next_epoch_protocol_version: ProtocolVersion,
         header: BlockHeader,
         chunks: Vec<ShardChunkHeader>,
         challenges: Challenges,
         vrf_value: near_crypto::vrf::Value,
         vrf_proof: near_crypto::vrf::Proof,
     ) -> Block {
-        if protocol_version < SHARD_CHUNK_HEADER_UPGRADE_VERSION {
+        if next_epoch_protocol_version < SHARD_CHUNK_HEADER_UPGRADE_VERSION {
             let legacy_chunks = chunks
                 .into_iter()
                 .map(|chunk| match chunk {
@@ -130,7 +134,6 @@ impl Block {
                     ShardChunkHeader::V2(_) => panic!(
                         "Attempted to include VersionedShardChunkHeaderV2 in old protocol version"
                     ),
-                    #[cfg(feature = "protocol_feature_block_header_v3")]
                     ShardChunkHeader::V3(_) => panic!(
                         "Attempted to include VersionedShardChunkHeaderV3 in old protocol version"
                     ),
@@ -192,16 +195,15 @@ impl Block {
 
     /// Produces new block from header of previous block, current state root and set of transactions.
     pub fn produce(
-        protocol_version: ProtocolVersion,
+        this_epoch_protocol_version: ProtocolVersion,
+        next_epoch_protocol_version: ProtocolVersion,
         prev: &BlockHeader,
         height: BlockHeight,
-        #[cfg(feature = "protocol_feature_block_header_v3")] block_ordinal: NumBlocks,
+        block_ordinal: NumBlocks,
         chunks: Vec<ShardChunkHeader>,
         epoch_id: EpochId,
         next_epoch_id: EpochId,
-        #[cfg(feature = "protocol_feature_block_header_v3")] epoch_sync_data_hash: Option<
-            CryptoHash,
-        >,
+        epoch_sync_data_hash: Option<CryptoHash>,
         approvals: Vec<Option<Signature>>,
         gas_price_adjustment_rate: Rational,
         min_gas_price: Balance,
@@ -257,7 +259,6 @@ impl Block {
                 prev.last_final_block()
             };
 
-        #[cfg(feature = "protocol_feature_block_header_v3")]
         match prev {
             BlockHeader::BlockHeaderV1(_) => debug_assert_eq!(prev.block_ordinal(), 0),
             BlockHeader::BlockHeaderV2(_) => debug_assert_eq!(prev.block_ordinal(), 0),
@@ -267,9 +268,10 @@ impl Block {
         };
 
         let header = BlockHeader::new(
-            protocol_version,
+            this_epoch_protocol_version,
+            next_epoch_protocol_version,
             height,
-            prev.hash().clone(),
+            *prev.hash(),
             Block::compute_state_root(&chunks),
             Block::compute_chunk_receipts_root(&chunks),
             Block::compute_chunk_headers_root(&chunks).0,
@@ -280,7 +282,6 @@ impl Block {
             random_value,
             validator_proposals,
             chunk_mask,
-            #[cfg(feature = "protocol_feature_block_header_v3")]
             block_ordinal,
             epoch_id,
             next_epoch_id,
@@ -288,19 +289,17 @@ impl Block {
             new_total_supply,
             challenges_result,
             signer,
-            last_final_block.clone(),
-            last_ds_final_block.clone(),
-            #[cfg(feature = "protocol_feature_block_header_v3")]
+            *last_final_block,
+            *last_ds_final_block,
             epoch_sync_data_hash,
             approvals,
             next_bp_hash,
             block_merkle_root,
-            #[cfg(feature = "protocol_feature_block_header_v3")]
             prev.height(),
         );
 
         Self::block_from_protocol_version(
-            protocol_version,
+            next_epoch_protocol_version,
             header,
             chunks,
             challenges,
@@ -524,7 +523,7 @@ impl Block {
         }
 
         // Check that challenges root stored in the header matches the challenges root of the challenges
-        let challenges_root = Block::compute_challenges_root(&self.challenges());
+        let challenges_root = Block::compute_challenges_root(self.challenges());
         if self.header().challenges_root() != &challenges_root {
             return Err(InvalidChallengeRoot);
         }
@@ -585,7 +584,7 @@ impl<'a> ChunksCollection<'a> {
 
     pub fn iter<'b: 'a>(&'b self) -> VersionedChunksIter<'b> {
         match self {
-            ChunksCollection::V1(chunks) => VersionedChunksIter::new(&chunks),
+            ChunksCollection::V1(chunks) => VersionedChunksIter::new(chunks),
             ChunksCollection::V2(chunks) => VersionedChunksIter::new(chunks),
         }
     }
@@ -620,8 +619,8 @@ impl Tip {
     pub fn from_header(header: &BlockHeader) -> Tip {
         Tip {
             height: header.height(),
-            last_block_hash: header.hash().clone(),
-            prev_block_hash: header.prev_hash().clone(),
+            last_block_hash: *header.hash(),
+            prev_block_hash: *header.prev_hash(),
             epoch_id: header.epoch_id().clone(),
             next_epoch_id: header.next_epoch_id().clone(),
         }
