@@ -15,6 +15,7 @@ use crate::utils::{from_timestamp, to_timestamp};
 use crate::validator_signer::ValidatorSigner;
 use crate::version::{ProtocolVersion, PROTOCOL_VERSION};
 
+#[cfg_attr(feature = "deepsize_feature", derive(deepsize::DeepSizeOf))]
 #[derive(BorshSerialize, BorshDeserialize, Serialize, Debug, Clone, Eq, PartialEq)]
 pub struct BlockHeaderInnerLite {
     /// Height of this block.
@@ -35,6 +36,7 @@ pub struct BlockHeaderInnerLite {
     pub block_merkle_root: CryptoHash,
 }
 
+#[cfg_attr(feature = "deepsize_feature", derive(deepsize::DeepSizeOf))]
 #[derive(BorshSerialize, BorshDeserialize, Serialize, Debug, Clone, Eq, PartialEq)]
 pub struct BlockHeaderInnerRest {
     /// Root hash of the chunk receipts in the given block.
@@ -73,6 +75,7 @@ pub struct BlockHeaderInnerRest {
 }
 
 /// Remove `chunks_included` from V1
+#[cfg_attr(feature = "deepsize_feature", derive(deepsize::DeepSizeOf))]
 #[derive(BorshSerialize, BorshDeserialize, Serialize, Debug, Clone, Eq, PartialEq)]
 pub struct BlockHeaderInnerRestV2 {
     /// Root hash of the chunk receipts in the given block.
@@ -112,7 +115,7 @@ pub struct BlockHeaderInnerRestV2 {
 /// Add `block_ordinal`
 /// Add `epoch_sync_data_hash`
 /// Use new `ValidatorStake` struct
-#[cfg(feature = "protocol_feature_block_header_v3")]
+#[cfg_attr(feature = "deepsize_feature", derive(deepsize::DeepSizeOf))]
 #[derive(BorshSerialize, BorshDeserialize, Serialize, Debug, Clone, Eq, PartialEq)]
 pub struct BlockHeaderInnerRestV3 {
     /// Root hash of the chunk receipts in the given block.
@@ -156,6 +159,7 @@ pub struct BlockHeaderInnerRestV3 {
 }
 
 /// The part of the block approval that is different for endorsements and skips
+#[cfg_attr(feature = "deepsize_feature", derive(deepsize::DeepSizeOf))]
 #[derive(BorshSerialize, BorshDeserialize, Serialize, Debug, Clone, PartialEq, Eq, Hash)]
 pub enum ApprovalInner {
     Endorsement(CryptoHash),
@@ -163,6 +167,7 @@ pub enum ApprovalInner {
 }
 
 /// Block approval by other block producers with a signature
+#[cfg_attr(feature = "deepsize_feature", derive(deepsize::DeepSizeOf))]
 #[derive(BorshSerialize, BorshDeserialize, Serialize, Debug, Clone, PartialEq, Eq)]
 pub struct Approval {
     pub inner: ApprovalInner,
@@ -179,6 +184,7 @@ pub enum ApprovalType {
 }
 
 /// Block approval by other block producers.
+#[cfg_attr(feature = "deepsize_feature", derive(deepsize::DeepSizeOf))]
 #[derive(BorshSerialize, BorshDeserialize, Serialize, Debug, Clone, PartialEq, Eq)]
 pub struct ApprovalMessage {
     pub approval: Approval,
@@ -192,7 +198,7 @@ impl ApprovalInner {
         target_height: BlockHeight,
     ) -> Self {
         if target_height == parent_height + 1 {
-            ApprovalInner::Endorsement(parent_hash.clone())
+            ApprovalInner::Endorsement(*parent_hash)
         } else {
             ApprovalInner::Skip(parent_height)
         }
@@ -222,6 +228,7 @@ impl ApprovalMessage {
     }
 }
 
+#[cfg_attr(feature = "deepsize_feature", derive(deepsize::DeepSizeOf))]
 #[derive(BorshSerialize, BorshDeserialize, Serialize, Debug, Clone, Eq, PartialEq)]
 #[borsh_init(init)]
 pub struct BlockHeaderV1 {
@@ -251,6 +258,7 @@ impl BlockHeaderV1 {
 }
 
 /// V1 -> V2: Remove `chunks_included` from `inner_reset`
+#[cfg_attr(feature = "deepsize_feature", derive(deepsize::DeepSizeOf))]
 #[derive(BorshSerialize, BorshDeserialize, Serialize, Debug, Clone, Eq, PartialEq)]
 #[borsh_init(init)]
 pub struct BlockHeaderV2 {
@@ -271,7 +279,7 @@ pub struct BlockHeaderV2 {
 
 /// V2 -> V3: Add `prev_height` to `inner_rest` and use new `ValidatorStake`
 // Add `block_ordinal` to `inner_rest`
-#[cfg(feature = "protocol_feature_block_header_v3")]
+#[cfg_attr(feature = "deepsize_feature", derive(deepsize::DeepSizeOf))]
 #[derive(BorshSerialize, BorshDeserialize, Serialize, Debug, Clone, Eq, PartialEq)]
 #[borsh_init(init)]
 pub struct BlockHeaderV3 {
@@ -299,7 +307,7 @@ impl BlockHeaderV2 {
         );
     }
 }
-#[cfg(feature = "protocol_feature_block_header_v3")]
+
 impl BlockHeaderV3 {
     pub fn init(&mut self) {
         self.hash = BlockHeader::compute_hash(
@@ -312,11 +320,11 @@ impl BlockHeaderV3 {
 
 /// Versioned BlockHeader data structure.
 /// For each next version, document what are the changes between versions.
+#[cfg_attr(feature = "deepsize_feature", derive(deepsize::DeepSizeOf))]
 #[derive(BorshSerialize, BorshDeserialize, Serialize, Debug, Clone, Eq, PartialEq)]
 pub enum BlockHeader {
     BlockHeaderV1(Box<BlockHeaderV1>),
     BlockHeaderV2(Box<BlockHeaderV2>),
-    #[cfg(feature = "protocol_feature_block_header_v3")]
     BlockHeaderV3(Box<BlockHeaderV3>),
 }
 
@@ -324,17 +332,18 @@ impl BlockHeader {
     pub fn compute_inner_hash(inner_lite: &[u8], inner_rest: &[u8]) -> CryptoHash {
         let hash_lite = hash(inner_lite);
         let hash_rest = hash(inner_rest);
-        combine_hash(hash_lite, hash_rest)
+        combine_hash(&hash_lite, &hash_rest)
     }
 
     pub fn compute_hash(prev_hash: CryptoHash, inner_lite: &[u8], inner_rest: &[u8]) -> CryptoHash {
         let hash_inner = BlockHeader::compute_inner_hash(inner_lite, inner_rest);
 
-        return combine_hash(hash_inner, prev_hash);
+        combine_hash(&hash_inner, &prev_hash)
     }
 
     pub fn new(
-        protocol_version: ProtocolVersion,
+        this_epoch_protocol_version: ProtocolVersion,
+        next_epoch_protocol_version: ProtocolVersion,
         height: BlockHeight,
         prev_hash: CryptoHash,
         prev_state_root: MerkleHash,
@@ -347,7 +356,7 @@ impl BlockHeader {
         random_value: CryptoHash,
         validator_proposals: Vec<ValidatorStake>,
         chunk_mask: Vec<bool>,
-        #[cfg(feature = "protocol_feature_block_header_v3")] block_ordinal: NumBlocks,
+        block_ordinal: NumBlocks,
         epoch_id: EpochId,
         next_epoch_id: EpochId,
         gas_price: Balance,
@@ -356,13 +365,11 @@ impl BlockHeader {
         signer: &dyn ValidatorSigner,
         last_final_block: CryptoHash,
         last_ds_final_block: CryptoHash,
-        #[cfg(feature = "protocol_feature_block_header_v3")] epoch_sync_data_hash: Option<
-            CryptoHash,
-        >,
+        epoch_sync_data_hash: Option<CryptoHash>,
         approvals: Vec<Option<Signature>>,
         next_bp_hash: CryptoHash,
         block_merkle_root: CryptoHash,
-        #[cfg(feature = "protocol_feature_block_header_v3")] prev_height: BlockHeight,
+        prev_height: BlockHeight,
     ) -> Self {
         let inner_lite = BlockHeaderInnerLite {
             height,
@@ -374,12 +381,11 @@ impl BlockHeader {
             next_bp_hash,
             block_merkle_root,
         };
-        #[cfg(not(feature = "protocol_feature_block_header_v3"))]
-        let last_header_v2_version: Option<u32> = None;
-        #[cfg(feature = "protocol_feature_block_header_v3")]
         let last_header_v2_version =
-            Some(crate::version::ProtocolFeature::BlockHeaderV3.protocol_version() - 1);
-        if protocol_version <= 29 {
+            crate::version::ProtocolFeature::BlockHeaderV3.protocol_version() - 1;
+        // Previously we passed next_epoch_protocol_version here, which is incorrect, but we need
+        // to preserve this for archival nodes
+        if next_epoch_protocol_version <= 29 {
             let chunks_included = chunk_mask.iter().map(|val| *val as u64).sum::<u64>();
             let inner_rest = BlockHeaderInnerRest {
                 chunk_receipts_root,
@@ -388,10 +394,7 @@ impl BlockHeader {
                 chunks_included,
                 challenges_root,
                 random_value,
-                #[cfg(feature = "protocol_feature_block_header_v3")]
                 validator_proposals: validator_proposals.into_iter().map(|v| v.into_v1()).collect(),
-                #[cfg(not(feature = "protocol_feature_block_header_v3"))]
-                validator_proposals,
                 chunk_mask,
                 gas_price,
                 total_supply,
@@ -413,18 +416,13 @@ impl BlockHeader {
                 signature,
                 hash,
             }))
-        } else if last_header_v2_version.is_none()
-            || protocol_version <= last_header_v2_version.unwrap()
-        {
+        } else if this_epoch_protocol_version <= last_header_v2_version {
             let inner_rest = BlockHeaderInnerRestV2 {
                 chunk_receipts_root,
                 chunk_headers_root,
                 chunk_tx_root,
                 challenges_root,
                 random_value,
-                #[cfg(not(feature = "protocol_feature_block_header_v3"))]
-                validator_proposals,
-                #[cfg(feature = "protocol_feature_block_header_v3")]
                 validator_proposals: validator_proposals.into_iter().map(|v| v.into_v1()).collect(),
                 chunk_mask,
                 gas_price,
@@ -448,42 +446,37 @@ impl BlockHeader {
                 hash,
             }))
         } else {
-            #[cfg(not(feature = "protocol_feature_block_header_v3"))]
-            unreachable!();
-            #[cfg(feature = "protocol_feature_block_header_v3")]
-            {
-                let inner_rest = BlockHeaderInnerRestV3 {
-                    chunk_receipts_root,
-                    chunk_headers_root,
-                    chunk_tx_root,
-                    challenges_root,
-                    random_value,
-                    validator_proposals,
-                    chunk_mask,
-                    gas_price,
-                    block_ordinal,
-                    total_supply,
-                    challenges_result,
-                    last_final_block,
-                    last_ds_final_block,
-                    prev_height,
-                    epoch_sync_data_hash,
-                    approvals,
-                    latest_protocol_version: PROTOCOL_VERSION,
-                };
-                let (hash, signature) = signer.sign_block_header_parts(
-                    prev_hash,
-                    &inner_lite.try_to_vec().expect("Failed to serialize"),
-                    &inner_rest.try_to_vec().expect("Failed to serialize"),
-                );
-                Self::BlockHeaderV3(Box::new(BlockHeaderV3 {
-                    prev_hash,
-                    inner_lite,
-                    inner_rest,
-                    signature,
-                    hash,
-                }))
-            }
+            let inner_rest = BlockHeaderInnerRestV3 {
+                chunk_receipts_root,
+                chunk_headers_root,
+                chunk_tx_root,
+                challenges_root,
+                random_value,
+                validator_proposals,
+                chunk_mask,
+                gas_price,
+                block_ordinal,
+                total_supply,
+                challenges_result,
+                last_final_block,
+                last_ds_final_block,
+                prev_height,
+                epoch_sync_data_hash,
+                approvals,
+                latest_protocol_version: PROTOCOL_VERSION,
+            };
+            let (hash, signature) = signer.sign_block_header_parts(
+                prev_hash,
+                &inner_lite.try_to_vec().expect("Failed to serialize"),
+                &inner_rest.try_to_vec().expect("Failed to serialize"),
+            );
+            Self::BlockHeaderV3(Box::new(BlockHeaderV3 {
+                prev_hash,
+                inner_lite,
+                inner_rest,
+                signature,
+                hash,
+            }))
         }
     }
 
@@ -512,11 +505,8 @@ impl BlockHeader {
             next_bp_hash,
             block_merkle_root: CryptoHash::default(),
         };
-        #[cfg(not(feature = "protocol_feature_block_header_v3"))]
-        let last_header_v2_version: Option<u32> = None;
-        #[cfg(feature = "protocol_feature_block_header_v3")]
         let last_header_v2_version =
-            Some(crate::version::ProtocolFeature::BlockHeaderV3.protocol_version() - 1);
+            crate::version::ProtocolFeature::BlockHeaderV3.protocol_version() - 1;
         if genesis_protocol_version <= 29 {
             let inner_rest = BlockHeaderInnerRest {
                 chunk_receipts_root,
@@ -547,9 +537,7 @@ impl BlockHeader {
                 signature: Signature::empty(KeyType::ED25519),
                 hash,
             }))
-        } else if last_header_v2_version.is_none()
-            || genesis_protocol_version <= last_header_v2_version.unwrap()
-        {
+        } else if genesis_protocol_version <= last_header_v2_version {
             let inner_rest = BlockHeaderInnerRestV2 {
                 chunk_receipts_root,
                 chunk_headers_root,
@@ -579,42 +567,37 @@ impl BlockHeader {
                 hash,
             }))
         } else {
-            #[cfg(not(feature = "protocol_feature_block_header_v3"))]
-            unreachable!();
-            #[cfg(feature = "protocol_feature_block_header_v3")]
-            {
-                let inner_rest = BlockHeaderInnerRestV3 {
-                    chunk_receipts_root,
-                    chunk_headers_root,
-                    chunk_tx_root,
-                    challenges_root,
-                    random_value: CryptoHash::default(),
-                    validator_proposals: vec![],
-                    chunk_mask: vec![true; chunks_included as usize],
-                    block_ordinal: 1, // It is guaranteed that Chain has the only Block which is Genesis
-                    gas_price: initial_gas_price,
-                    total_supply: initial_total_supply,
-                    challenges_result: vec![],
-                    last_final_block: CryptoHash::default(),
-                    last_ds_final_block: CryptoHash::default(),
-                    prev_height: 0,
-                    epoch_sync_data_hash: None, // Epoch Sync cannot be executed up to Genesis
-                    approvals: vec![],
-                    latest_protocol_version: genesis_protocol_version,
-                };
-                let hash = BlockHeader::compute_hash(
-                    CryptoHash::default(),
-                    &inner_lite.try_to_vec().expect("Failed to serialize"),
-                    &inner_rest.try_to_vec().expect("Failed to serialize"),
-                );
-                Self::BlockHeaderV3(Box::new(BlockHeaderV3 {
-                    prev_hash: CryptoHash::default(),
-                    inner_lite,
-                    inner_rest,
-                    signature: Signature::empty(KeyType::ED25519),
-                    hash,
-                }))
-            }
+            let inner_rest = BlockHeaderInnerRestV3 {
+                chunk_receipts_root,
+                chunk_headers_root,
+                chunk_tx_root,
+                challenges_root,
+                random_value: CryptoHash::default(),
+                validator_proposals: vec![],
+                chunk_mask: vec![true; chunks_included as usize],
+                block_ordinal: 1, // It is guaranteed that Chain has the only Block which is Genesis
+                gas_price: initial_gas_price,
+                total_supply: initial_total_supply,
+                challenges_result: vec![],
+                last_final_block: CryptoHash::default(),
+                last_ds_final_block: CryptoHash::default(),
+                prev_height: 0,
+                epoch_sync_data_hash: None, // Epoch Sync cannot be executed up to Genesis
+                approvals: vec![],
+                latest_protocol_version: genesis_protocol_version,
+            };
+            let hash = BlockHeader::compute_hash(
+                CryptoHash::default(),
+                &inner_lite.try_to_vec().expect("Failed to serialize"),
+                &inner_rest.try_to_vec().expect("Failed to serialize"),
+            );
+            Self::BlockHeaderV3(Box::new(BlockHeaderV3 {
+                prev_hash: CryptoHash::default(),
+                inner_lite,
+                inner_rest,
+                signature: Signature::empty(KeyType::ED25519),
+                hash,
+            }))
         }
     }
 
@@ -623,7 +606,6 @@ impl BlockHeader {
         match self {
             BlockHeader::BlockHeaderV1(header) => &header.hash,
             BlockHeader::BlockHeaderV2(header) => &header.hash,
-            #[cfg(feature = "protocol_feature_block_header_v3")]
             BlockHeader::BlockHeaderV3(header) => &header.hash,
         }
     }
@@ -633,7 +615,6 @@ impl BlockHeader {
         match self {
             BlockHeader::BlockHeaderV1(header) => &header.prev_hash,
             BlockHeader::BlockHeaderV2(header) => &header.prev_hash,
-            #[cfg(feature = "protocol_feature_block_header_v3")]
             BlockHeader::BlockHeaderV3(header) => &header.prev_hash,
         }
     }
@@ -643,7 +624,6 @@ impl BlockHeader {
         match self {
             BlockHeader::BlockHeaderV1(header) => &header.signature,
             BlockHeader::BlockHeaderV2(header) => &header.signature,
-            #[cfg(feature = "protocol_feature_block_header_v3")]
             BlockHeader::BlockHeaderV3(header) => &header.signature,
         }
     }
@@ -653,13 +633,11 @@ impl BlockHeader {
         match self {
             BlockHeader::BlockHeaderV1(header) => header.inner_lite.height,
             BlockHeader::BlockHeaderV2(header) => header.inner_lite.height,
-            #[cfg(feature = "protocol_feature_block_header_v3")]
             BlockHeader::BlockHeaderV3(header) => header.inner_lite.height,
         }
     }
 
     #[inline]
-    #[cfg(feature = "protocol_feature_block_header_v3")]
     pub fn prev_height(&self) -> Option<BlockHeight> {
         match self {
             BlockHeader::BlockHeaderV1(_) => None,
@@ -673,7 +651,6 @@ impl BlockHeader {
         match self {
             BlockHeader::BlockHeaderV1(header) => &header.inner_lite.epoch_id,
             BlockHeader::BlockHeaderV2(header) => &header.inner_lite.epoch_id,
-            #[cfg(feature = "protocol_feature_block_header_v3")]
             BlockHeader::BlockHeaderV3(header) => &header.inner_lite.epoch_id,
         }
     }
@@ -683,7 +660,6 @@ impl BlockHeader {
         match self {
             BlockHeader::BlockHeaderV1(header) => &header.inner_lite.next_epoch_id,
             BlockHeader::BlockHeaderV2(header) => &header.inner_lite.next_epoch_id,
-            #[cfg(feature = "protocol_feature_block_header_v3")]
             BlockHeader::BlockHeaderV3(header) => &header.inner_lite.next_epoch_id,
         }
     }
@@ -693,7 +669,6 @@ impl BlockHeader {
         match self {
             BlockHeader::BlockHeaderV1(header) => &header.inner_lite.prev_state_root,
             BlockHeader::BlockHeaderV2(header) => &header.inner_lite.prev_state_root,
-            #[cfg(feature = "protocol_feature_block_header_v3")]
             BlockHeader::BlockHeaderV3(header) => &header.inner_lite.prev_state_root,
         }
     }
@@ -703,7 +678,6 @@ impl BlockHeader {
         match self {
             BlockHeader::BlockHeaderV1(header) => &header.inner_rest.chunk_receipts_root,
             BlockHeader::BlockHeaderV2(header) => &header.inner_rest.chunk_receipts_root,
-            #[cfg(feature = "protocol_feature_block_header_v3")]
             BlockHeader::BlockHeaderV3(header) => &header.inner_rest.chunk_receipts_root,
         }
     }
@@ -713,7 +687,6 @@ impl BlockHeader {
         match self {
             BlockHeader::BlockHeaderV1(header) => &header.inner_rest.chunk_headers_root,
             BlockHeader::BlockHeaderV2(header) => &header.inner_rest.chunk_headers_root,
-            #[cfg(feature = "protocol_feature_block_header_v3")]
             BlockHeader::BlockHeaderV3(header) => &header.inner_rest.chunk_headers_root,
         }
     }
@@ -723,7 +696,6 @@ impl BlockHeader {
         match self {
             BlockHeader::BlockHeaderV1(header) => &header.inner_rest.chunk_tx_root,
             BlockHeader::BlockHeaderV2(header) => &header.inner_rest.chunk_tx_root,
-            #[cfg(feature = "protocol_feature_block_header_v3")]
             BlockHeader::BlockHeaderV3(header) => &header.inner_rest.chunk_tx_root,
         }
     }
@@ -734,7 +706,6 @@ impl BlockHeader {
             BlockHeader::BlockHeaderV2(header) => {
                 header.inner_rest.chunk_mask.iter().map(|&x| u64::from(x)).sum::<u64>()
             }
-            #[cfg(feature = "protocol_feature_block_header_v3")]
             BlockHeader::BlockHeaderV3(header) => {
                 header.inner_rest.chunk_mask.iter().map(|&x| u64::from(x)).sum::<u64>()
             }
@@ -746,7 +717,6 @@ impl BlockHeader {
         match self {
             BlockHeader::BlockHeaderV1(header) => &header.inner_rest.challenges_root,
             BlockHeader::BlockHeaderV2(header) => &header.inner_rest.challenges_root,
-            #[cfg(feature = "protocol_feature_block_header_v3")]
             BlockHeader::BlockHeaderV3(header) => &header.inner_rest.challenges_root,
         }
     }
@@ -756,7 +726,6 @@ impl BlockHeader {
         match self {
             BlockHeader::BlockHeaderV1(header) => &header.inner_lite.outcome_root,
             BlockHeader::BlockHeaderV2(header) => &header.inner_lite.outcome_root,
-            #[cfg(feature = "protocol_feature_block_header_v3")]
             BlockHeader::BlockHeaderV3(header) => &header.inner_lite.outcome_root,
         }
     }
@@ -766,7 +735,6 @@ impl BlockHeader {
         match self {
             BlockHeader::BlockHeaderV1(header) => header.inner_lite.timestamp,
             BlockHeader::BlockHeaderV2(header) => header.inner_lite.timestamp,
-            #[cfg(feature = "protocol_feature_block_header_v3")]
             BlockHeader::BlockHeaderV3(header) => header.inner_lite.timestamp,
         }
     }
@@ -780,7 +748,6 @@ impl BlockHeader {
             BlockHeader::BlockHeaderV2(header) => {
                 ValidatorStakeIter::v1(&header.inner_rest.validator_proposals)
             }
-            #[cfg(feature = "protocol_feature_block_header_v3")]
             BlockHeader::BlockHeaderV3(header) => {
                 ValidatorStakeIter::new(&header.inner_rest.validator_proposals)
             }
@@ -792,7 +759,6 @@ impl BlockHeader {
         match self {
             BlockHeader::BlockHeaderV1(header) => &header.inner_rest.chunk_mask,
             BlockHeader::BlockHeaderV2(header) => &header.inner_rest.chunk_mask,
-            #[cfg(feature = "protocol_feature_block_header_v3")]
             BlockHeader::BlockHeaderV3(header) => &header.inner_rest.chunk_mask,
         }
     }
@@ -802,7 +768,6 @@ impl BlockHeader {
         match self {
             BlockHeader::BlockHeaderV1(_) => 0, // not applicable
             BlockHeader::BlockHeaderV2(_) => 0, // not applicable
-            #[cfg(feature = "protocol_feature_block_header_v3")]
             BlockHeader::BlockHeaderV3(header) => header.inner_rest.block_ordinal,
         }
     }
@@ -812,7 +777,6 @@ impl BlockHeader {
         match self {
             BlockHeader::BlockHeaderV1(header) => header.inner_rest.gas_price,
             BlockHeader::BlockHeaderV2(header) => header.inner_rest.gas_price,
-            #[cfg(feature = "protocol_feature_block_header_v3")]
             BlockHeader::BlockHeaderV3(header) => header.inner_rest.gas_price,
         }
     }
@@ -822,7 +786,6 @@ impl BlockHeader {
         match self {
             BlockHeader::BlockHeaderV1(header) => header.inner_rest.total_supply,
             BlockHeader::BlockHeaderV2(header) => header.inner_rest.total_supply,
-            #[cfg(feature = "protocol_feature_block_header_v3")]
             BlockHeader::BlockHeaderV3(header) => header.inner_rest.total_supply,
         }
     }
@@ -832,7 +795,6 @@ impl BlockHeader {
         match self {
             BlockHeader::BlockHeaderV1(header) => &header.inner_rest.random_value,
             BlockHeader::BlockHeaderV2(header) => &header.inner_rest.random_value,
-            #[cfg(feature = "protocol_feature_block_header_v3")]
             BlockHeader::BlockHeaderV3(header) => &header.inner_rest.random_value,
         }
     }
@@ -842,7 +804,6 @@ impl BlockHeader {
         match self {
             BlockHeader::BlockHeaderV1(header) => &header.inner_rest.last_final_block,
             BlockHeader::BlockHeaderV2(header) => &header.inner_rest.last_final_block,
-            #[cfg(feature = "protocol_feature_block_header_v3")]
             BlockHeader::BlockHeaderV3(header) => &header.inner_rest.last_final_block,
         }
     }
@@ -852,7 +813,6 @@ impl BlockHeader {
         match self {
             BlockHeader::BlockHeaderV1(header) => &header.inner_rest.last_ds_final_block,
             BlockHeader::BlockHeaderV2(header) => &header.inner_rest.last_ds_final_block,
-            #[cfg(feature = "protocol_feature_block_header_v3")]
             BlockHeader::BlockHeaderV3(header) => &header.inner_rest.last_ds_final_block,
         }
     }
@@ -862,7 +822,6 @@ impl BlockHeader {
         match self {
             BlockHeader::BlockHeaderV1(header) => &header.inner_rest.challenges_result,
             BlockHeader::BlockHeaderV2(header) => &header.inner_rest.challenges_result,
-            #[cfg(feature = "protocol_feature_block_header_v3")]
             BlockHeader::BlockHeaderV3(header) => &header.inner_rest.challenges_result,
         }
     }
@@ -872,7 +831,6 @@ impl BlockHeader {
         match self {
             BlockHeader::BlockHeaderV1(header) => &header.inner_lite.next_bp_hash,
             BlockHeader::BlockHeaderV2(header) => &header.inner_lite.next_bp_hash,
-            #[cfg(feature = "protocol_feature_block_header_v3")]
             BlockHeader::BlockHeaderV3(header) => &header.inner_lite.next_bp_hash,
         }
     }
@@ -882,7 +840,6 @@ impl BlockHeader {
         match self {
             BlockHeader::BlockHeaderV1(header) => &header.inner_lite.block_merkle_root,
             BlockHeader::BlockHeaderV2(header) => &header.inner_lite.block_merkle_root,
-            #[cfg(feature = "protocol_feature_block_header_v3")]
             BlockHeader::BlockHeaderV3(header) => &header.inner_lite.block_merkle_root,
         }
     }
@@ -892,7 +849,6 @@ impl BlockHeader {
         match self {
             BlockHeader::BlockHeaderV1(_) => None,
             BlockHeader::BlockHeaderV2(_) => None,
-            #[cfg(feature = "protocol_feature_block_header_v3")]
             BlockHeader::BlockHeaderV3(header) => header.inner_rest.epoch_sync_data_hash,
         }
     }
@@ -902,7 +858,6 @@ impl BlockHeader {
         match self {
             BlockHeader::BlockHeaderV1(header) => &header.inner_rest.approvals,
             BlockHeader::BlockHeaderV2(header) => &header.inner_rest.approvals,
-            #[cfg(feature = "protocol_feature_block_header_v3")]
             BlockHeader::BlockHeaderV3(header) => &header.inner_rest.approvals,
         }
     }
@@ -927,7 +882,6 @@ impl BlockHeader {
                     == header.inner_rest.chunks_included
             }
             BlockHeader::BlockHeaderV2(_header) => true,
-            #[cfg(feature = "protocol_feature_block_header_v3")]
             BlockHeader::BlockHeaderV3(_header) => true,
         }
     }
@@ -937,7 +891,6 @@ impl BlockHeader {
         match self {
             BlockHeader::BlockHeaderV1(header) => header.inner_rest.latest_protocol_version,
             BlockHeader::BlockHeaderV2(header) => header.inner_rest.latest_protocol_version,
-            #[cfg(feature = "protocol_feature_block_header_v3")]
             BlockHeader::BlockHeaderV3(header) => header.inner_rest.latest_protocol_version,
         }
     }
@@ -950,7 +903,6 @@ impl BlockHeader {
             BlockHeader::BlockHeaderV2(header) => {
                 header.inner_lite.try_to_vec().expect("Failed to serialize")
             }
-            #[cfg(feature = "protocol_feature_block_header_v3")]
             BlockHeader::BlockHeaderV3(header) => {
                 header.inner_lite.try_to_vec().expect("Failed to serialize")
             }
@@ -965,7 +917,6 @@ impl BlockHeader {
             BlockHeader::BlockHeaderV2(header) => {
                 header.inner_rest.try_to_vec().expect("Failed to serialize")
             }
-            #[cfg(feature = "protocol_feature_block_header_v3")]
             BlockHeader::BlockHeaderV3(header) => {
                 header.inner_rest.try_to_vec().expect("Failed to serialize")
             }
