@@ -19,6 +19,7 @@ pub use near_primitives_core::types::*;
 pub type StateRoot = CryptoHash;
 
 /// Different types of finality.
+#[cfg_attr(feature = "deepsize_feature", derive(deepsize::DeepSizeOf))]
 #[derive(Serialize, Deserialize, BorshSerialize, BorshDeserialize, Clone, Debug, PartialEq, Eq)]
 pub enum Finality {
     #[serde(rename = "optimistic")]
@@ -54,6 +55,7 @@ pub struct AccountInfo {
 ///
 /// NOTE: Currently, this type is only used in the view_client and RPC to be able to transparently
 /// pretty-serialize the bytes arrays as base64-encoded strings (see `serialize.rs`).
+#[cfg_attr(feature = "deepsize_feature", derive(deepsize::DeepSizeOf))]
 #[derive(
     Debug, Clone, PartialEq, Eq, DeriveAsRef, DeriveFrom, BorshSerialize, BorshDeserialize,
 )]
@@ -75,6 +77,7 @@ pub struct StoreValue(Vec<u8>);
 /// NOTE: The main reason for this to exist (except the type-safety) is that the value is
 /// transparently serialized and deserialized as a base64-encoded string when serde is used
 /// (serde_json).
+#[cfg_attr(feature = "deepsize_feature", derive(deepsize::DeepSizeOf))]
 #[derive(
     Debug, Clone, PartialEq, Eq, DeriveAsRef, DeriveFrom, BorshSerialize, BorshDeserialize,
 )]
@@ -288,15 +291,14 @@ impl StateChanges {
                     state_changes.extend(changes.into_iter().map(
                         |RawStateChange { cause, data }| StateChangeWithCause {
                             cause,
-                            value: if let Some(change_data) = data {
-                                StateChangeValue::ContractCodeUpdate {
+                            value: match data {
+                                Some(change_data) => StateChangeValue::ContractCodeUpdate {
                                     account_id: account_id.clone(),
-                                    code: change_data.into(),
-                                }
-                            } else {
-                                StateChangeValue::ContractCodeDeletion {
+                                    code: change_data,
+                                },
+                                None => StateChangeValue::ContractCodeDeletion {
                                     account_id: account_id.clone(),
-                                }
+                                },
                             },
                         },
                     ));
@@ -400,6 +402,7 @@ impl StateChanges {
     }
 }
 
+#[cfg_attr(feature = "deepsize_feature", derive(deepsize::DeepSizeOf))]
 #[derive(PartialEq, Eq, Clone, Debug, BorshSerialize, BorshDeserialize, Serialize)]
 pub struct StateRootNode {
     /// in Nightshade, data is the serialized TrieNodeWithSize
@@ -417,6 +420,7 @@ impl StateRootNode {
 /// Epoch identifier -- wrapped hash, to make it easier to distinguish.
 /// EpochId of epoch T is the hash of last block in T-2
 /// EpochId of first two epochs is 0
+#[cfg_attr(feature = "deepsize_feature", derive(deepsize::DeepSizeOf))]
 #[derive(
     Debug,
     Clone,
@@ -437,6 +441,7 @@ pub struct EpochId(pub CryptoHash);
 /// Stores validator and its stake for two consecutive epochs.
 /// It is necessary because the blocks on the epoch boundary need to contain approvals from both
 /// epochs.
+#[cfg_attr(feature = "deepsize_feature", derive(deepsize::DeepSizeOf))]
 #[derive(BorshSerialize, BorshDeserialize, Serialize, Debug, Clone, PartialEq, Eq)]
 pub struct ApprovalStake {
     /// Account that stakes money.
@@ -448,7 +453,6 @@ pub struct ApprovalStake {
     pub stake_next_epoch: Balance,
 }
 
-#[cfg(feature = "protocol_feature_block_header_v3")]
 pub mod validator_stake {
     use crate::types::ApprovalStake;
     use borsh::{BorshDeserialize, BorshSerialize};
@@ -459,6 +463,7 @@ pub mod validator_stake {
     pub use super::ValidatorStakeV1;
 
     /// Stores validator and its stake.
+    #[cfg_attr(feature = "deepsize_feature", derive(deepsize::DeepSizeOf))]
     #[derive(BorshSerialize, BorshDeserialize, Serialize, Debug, Clone, PartialEq, Eq)]
     #[serde(tag = "validator_stake_struct_version")]
     pub enum ValidatorStake {
@@ -467,6 +472,7 @@ pub mod validator_stake {
         V2(ValidatorStakeV2),
     }
 
+    #[cfg_attr(feature = "deepsize_feature", derive(deepsize::DeepSizeOf))]
     #[cfg(feature = "protocol_feature_chunk_only_producers")]
     #[derive(BorshSerialize, BorshDeserialize, Serialize, Debug, Clone, PartialEq, Eq)]
     pub struct ValidatorStakeV2 {
@@ -562,9 +568,9 @@ pub mod validator_stake {
                 Self::V1(v1) => v1,
                 #[cfg(feature = "protocol_feature_chunk_only_producers")]
                 Self::V2(v2) => {
-                    // This function should never be called on a V2 variant, it
-                    // is only for backwards compatibility purposes.
-                    debug_assert!(false);
+                    // This function is called on V2 variant if
+                    // protocol_feature_chunk_only_producers is enabled, but current protocol
+                    // version is lower than required for block headers v3.
                     ValidatorStakeV1 {
                         account_id: v2.account_id,
                         public_key: v2.public_key,
@@ -574,11 +580,11 @@ pub mod validator_stake {
             }
         }
 
-        #[cfg(feature = "protocol_feature_chunk_only_producers")]
         #[inline]
         pub fn is_chunk_only(&self) -> bool {
             match self {
                 Self::V1(_) => false,
+                #[cfg(feature = "protocol_feature_chunk_only_producers")]
                 Self::V2(v2) => v2.is_chunk_only,
             }
         }
@@ -666,108 +672,8 @@ pub mod validator_stake {
     }
 }
 
-#[cfg(not(feature = "protocol_feature_block_header_v3"))]
-pub mod validator_stake {
-    use crate::types::ApprovalStake;
-    use near_crypto::PublicKey;
-    use near_primitives_core::types::{AccountId, Balance};
-
-    pub use super::ValidatorStakeV1;
-    pub type ValidatorStake = ValidatorStakeV1;
-
-    pub struct ValidatorStakeIter<'a> {
-        inner: std::slice::Iter<'a, ValidatorStake>,
-        len: usize,
-    }
-
-    impl<'a> ValidatorStakeIter<'a> {
-        pub fn empty() -> Self {
-            Self { inner: (&[]).iter(), len: 0 }
-        }
-
-        pub fn v1(collection: &'a [ValidatorStakeV1]) -> Self {
-            Self::new(collection)
-        }
-
-        pub fn new(collection: &'a [ValidatorStake]) -> Self {
-            Self { inner: collection.iter(), len: collection.len() }
-        }
-
-        pub fn len(&self) -> usize {
-            self.len
-        }
-    }
-
-    impl<'a> Iterator for ValidatorStakeIter<'a> {
-        type Item = ValidatorStake;
-
-        fn next(&mut self) -> Option<Self::Item> {
-            self.inner.next().cloned()
-        }
-    }
-
-    impl ValidatorStake {
-        pub fn new(account_id: AccountId, public_key: PublicKey, stake: Balance) -> Self {
-            ValidatorStake { account_id, public_key, stake }
-        }
-
-        #[inline]
-        pub fn account_and_stake(self) -> (AccountId, Balance) {
-            (self.account_id, self.stake)
-        }
-
-        #[inline]
-        pub fn destructure(self) -> (AccountId, PublicKey, Balance) {
-            (self.account_id, self.public_key, self.stake)
-        }
-
-        #[inline]
-        pub fn into_v1(self) -> ValidatorStakeV1 {
-            self
-        }
-
-        #[inline]
-        pub fn take_account_id(self) -> AccountId {
-            self.account_id
-        }
-
-        #[inline]
-        pub fn account_id(&self) -> &AccountId {
-            &self.account_id
-        }
-
-        #[inline]
-        pub fn take_public_key(self) -> PublicKey {
-            self.public_key
-        }
-
-        #[inline]
-        pub fn public_key(&self) -> &PublicKey {
-            &self.public_key
-        }
-
-        #[inline]
-        pub fn stake(&self) -> Balance {
-            self.stake
-        }
-
-        #[inline]
-        pub fn stake_mut(&mut self) -> &mut Balance {
-            &mut self.stake
-        }
-
-        pub fn get_approval_stake(&self, is_next_epoch: bool) -> ApprovalStake {
-            ApprovalStake {
-                account_id: self.account_id.clone(),
-                public_key: self.public_key.clone(),
-                stake_this_epoch: if is_next_epoch { 0 } else { self.stake },
-                stake_next_epoch: if is_next_epoch { self.stake } else { 0 },
-            }
-        }
-    }
-}
-
 /// Stores validator and its stake.
+#[cfg_attr(feature = "deepsize_feature", derive(deepsize::DeepSizeOf))]
 #[derive(BorshSerialize, BorshDeserialize, Serialize, Debug, Clone, PartialEq, Eq)]
 pub struct ValidatorStakeV1 {
     /// Account that stakes money.
@@ -779,12 +685,12 @@ pub struct ValidatorStakeV1 {
 }
 
 /// Information after block was processed.
+#[cfg_attr(feature = "deepsize_feature", derive(deepsize::DeepSizeOf))]
 #[derive(Debug, PartialEq, BorshSerialize, BorshDeserialize, Clone, Eq)]
 pub struct BlockExtra {
     pub challenges_result: ChallengesResult,
 }
 
-#[cfg(feature = "protocol_feature_block_header_v3")]
 pub mod chunk_extra {
     use crate::types::validator_stake::{ValidatorStake, ValidatorStakeIter};
     use crate::types::StateRoot;
@@ -831,7 +737,7 @@ pub mod chunk_extra {
             balance_burnt: Balance,
         ) -> Self {
             Self::V2(ChunkExtraV2 {
-                state_root: state_root.clone(),
+                state_root: *state_root,
                 outcome_root,
                 validator_proposals,
                 gas_used,
@@ -898,76 +804,6 @@ pub mod chunk_extra {
     }
 }
 
-#[cfg(not(feature = "protocol_feature_block_header_v3"))]
-pub mod chunk_extra {
-    use crate::types::validator_stake::{ValidatorStake, ValidatorStakeIter};
-    use crate::types::StateRoot;
-    use near_primitives_core::hash::CryptoHash;
-    use near_primitives_core::types::{Balance, Gas};
-
-    pub use super::ChunkExtraV1;
-    pub type ChunkExtra = ChunkExtraV1;
-
-    impl ChunkExtra {
-        pub fn new_with_only_state_root(state_root: &StateRoot) -> Self {
-            Self::new(state_root, CryptoHash::default(), vec![], 0, 0, 0)
-        }
-
-        pub fn new(
-            state_root: &StateRoot,
-            outcome_root: CryptoHash,
-            validator_proposals: Vec<ValidatorStake>,
-            gas_used: Gas,
-            gas_limit: Gas,
-            balance_burnt: Balance,
-        ) -> Self {
-            Self {
-                state_root: state_root.clone(),
-                outcome_root,
-                validator_proposals,
-                gas_used,
-                gas_limit,
-                balance_burnt,
-            }
-        }
-
-        #[inline]
-        pub fn outcome_root(&self) -> &StateRoot {
-            &self.outcome_root
-        }
-
-        #[inline]
-        pub fn state_root(&self) -> &StateRoot {
-            &self.state_root
-        }
-
-        #[inline]
-        pub fn state_root_mut(&mut self) -> &mut StateRoot {
-            &mut self.state_root
-        }
-
-        #[inline]
-        pub fn validator_proposals(&self) -> ValidatorStakeIter {
-            ValidatorStakeIter::new(&self.validator_proposals)
-        }
-
-        #[inline]
-        pub fn gas_limit(&self) -> Gas {
-            self.gas_limit
-        }
-
-        #[inline]
-        pub fn gas_used(&self) -> Gas {
-            self.gas_used
-        }
-
-        #[inline]
-        pub fn balance_burnt(&self) -> Balance {
-            self.balance_burnt
-        }
-    }
-}
-
 /// Information after chunk was processed, used to produce or check next chunk.
 #[derive(Debug, PartialEq, BorshSerialize, BorshDeserialize, Clone, Eq)]
 pub struct ChunkExtraV1 {
@@ -985,6 +821,7 @@ pub struct ChunkExtraV1 {
     pub balance_burnt: Balance,
 }
 
+#[cfg_attr(feature = "deepsize_feature", derive(deepsize::DeepSizeOf))]
 #[derive(Debug, Clone, PartialEq, Eq, BorshSerialize, BorshDeserialize, Serialize, Deserialize)]
 #[serde(untagged)]
 pub enum BlockId {
@@ -994,6 +831,7 @@ pub enum BlockId {
 
 pub type MaybeBlockId = Option<BlockId>;
 
+#[cfg_attr(feature = "deepsize_feature", derive(deepsize::DeepSizeOf))]
 #[derive(Debug, Clone, PartialEq, Eq, BorshSerialize, BorshDeserialize, Serialize, Deserialize)]
 #[serde(rename_all = "snake_case")]
 pub enum SyncCheckpoint {
@@ -1001,6 +839,7 @@ pub enum SyncCheckpoint {
     EarliestAvailable,
 }
 
+#[cfg_attr(feature = "deepsize_feature", derive(deepsize::DeepSizeOf))]
 #[derive(Debug, Clone, PartialEq, Eq, BorshSerialize, BorshDeserialize, Serialize, Deserialize)]
 #[serde(rename_all = "snake_case")]
 pub enum BlockReference {
@@ -1067,6 +906,7 @@ impl Serialize for EpochReference {
 }
 
 /// Reasons for removing a validator from the validator set.
+#[cfg_attr(feature = "deepsize_feature", derive(deepsize::DeepSizeOf))]
 #[derive(BorshSerialize, BorshDeserialize, Serialize, Deserialize, Clone, Debug, PartialEq, Eq)]
 pub enum ValidatorKickoutReason {
     /// Slashed validators are kicked out.
