@@ -2,7 +2,6 @@ use std::cmp::max;
 use std::convert::AsRef;
 use std::fmt;
 
-use byteorder::{LittleEndian, WriteBytesExt};
 use chrono;
 use chrono::{DateTime, NaiveDateTime};
 use rand::distributions::Alphanumeric;
@@ -62,7 +61,17 @@ pub struct MaybeValidated<T> {
 impl<T> MaybeValidated<T> {
     /// Creates new MaybeValidated object marking payload as validated.  No
     /// verification is performed; it’s caller’s responsibility to make sure the
-    /// payload was indeed validated.
+    /// payload has indeed been validated.
+    ///
+    /// # Example
+    ///
+    /// ```
+    /// use near_primitives::utils::MaybeValidated;
+    ///
+    /// let value = MaybeValidated::from_validated(42);
+    /// assert!(value.is_validated());
+    /// assert_eq!(Ok(true), value.validate_with::<(), _>(|_| panic!()));
+    /// ```
     pub fn from_validated(payload: T) -> Self {
         Self { validated: std::cell::Cell::new(true), payload }
     }
@@ -79,7 +88,11 @@ impl<T> MaybeValidated<T> {
     /// use near_primitives::utils::MaybeValidated;
     ///
     /// let value = MaybeValidated::from(42);
+    /// assert_eq!(Err(()), value.validate_with(|_| Err(())));
+    /// assert_eq!(Ok(false), value.validate_with::<(), _>(|v| Ok(*v == 24)));
+    /// assert!(!value.is_validated());
     /// assert_eq!(Ok(true), value.validate_with::<(), _>(|v| Ok(*v == 42)));
+    /// assert!(value.is_validated());
     /// assert_eq!(Ok(true), value.validate_with::<(), _>(|_| panic!()));
     /// ```
     pub fn validate_with<E, F: FnOnce(&T) -> Result<bool, E>>(
@@ -95,9 +108,24 @@ impl<T> MaybeValidated<T> {
         }
     }
 
+    /// Marks the payload as valid.  No verification is performed; it’s caller’s
+    /// responsibility to make sure the payload has indeed been validated.
+    pub fn mark_as_valid(&self) {
+        self.validated.set(true);
+    }
+
     /// Applies function to the payload (whether it’s been validated or not) and
     /// returns new object with result of the function as payload.  Validated
     /// state is not changed.
+    ///
+    /// # Example
+    ///
+    /// ```
+    /// use near_primitives::utils::MaybeValidated;
+    ///
+    /// let value = MaybeValidated::from(42);
+    /// assert_eq!("42", value.map(|v| v.to_string()).into_inner());
+    /// ```
     pub fn map<U, F: FnOnce(T) -> U>(self, validator: F) -> MaybeValidated<U> {
         MaybeValidated { validated: self.validated, payload: validator(self.payload) }
     }
@@ -105,14 +133,36 @@ impl<T> MaybeValidated<T> {
     /// Returns a new object storing reference to this object’s payload.  Note
     /// that the two objects do not share the validated state so calling
     /// `validate_with` on one of them does not affect the other.
+    ///
+    /// # Example
+    ///
+    /// ```
+    /// use near_primitives::utils::MaybeValidated;
+    ///
+    /// let value = MaybeValidated::from(42);
+    /// let value_as_ref = value.as_ref();
+    /// assert_eq!(Ok(true), value_as_ref.validate_with::<(), _>(|&&v| Ok(v == 42)));
+    /// assert!(value_as_ref.is_validated());
+    /// assert!(!value.is_validated());
+    /// ```
     pub fn as_ref(&self) -> MaybeValidated<&T> {
         MaybeValidated { validated: self.validated.clone(), payload: &self.payload }
     }
 
+    /// Returns whether the payload has been validated.
+    pub fn is_validated(&self) -> bool {
+        self.validated.get()
+    }
+
     /// Extracts the payload whether or not it’s been validated.
     pub fn into_inner(self) -> T {
         self.payload
     }
+
+    /// Returns a reference to the payload
+    pub fn get_inner(&self) -> &T {
+        &self.payload
+    }
 }
 
 impl<T> From<T> for MaybeValidated<T> {
@@ -145,8 +195,7 @@ pub fn get_block_shard_id_rev(
             std::io::Error::new(std::io::ErrorKind::InvalidInput, "Invalid key length").into()
         );
     }
-    let block_hash_vec: Vec<u8> = key[0..32].iter().cloned().collect();
-    let block_hash = CryptoHash::try_from(block_hash_vec)?;
+    let block_hash = CryptoHash::try_from(&key[..32])?;
     let mut shard_id_arr: [u8; 8] = Default::default();
     shard_id_arr.copy_from_slice(&key[key.len() - 8..]);
     let shard_id = ShardId::from_le_bytes(shard_id_arr);
@@ -164,8 +213,8 @@ pub fn create_receipt_id_from_transaction(
     create_hash_upgradable(
         protocol_version,
         &signed_transaction.get_hash(),
-        &prev_block_hash,
-        &block_hash,
+        prev_block_hash,
+        block_hash,
         0,
     )
 }
@@ -182,8 +231,8 @@ pub fn create_receipt_id_from_receipt(
     create_hash_upgradable(
         protocol_version,
         &receipt.receipt_id,
-        &prev_block_hash,
-        &block_hash,
+        prev_block_hash,
+        block_hash,
         receipt_index as u64,
     )
 }
@@ -200,13 +249,7 @@ pub fn create_action_hash(
     // Action hash uses the same input as a new receipt ID, so to avoid hash conflicts we use the
     // salt starting from the `u64` going backward.
     let salt = u64::max_value() - action_index as u64;
-    create_hash_upgradable(
-        protocol_version,
-        &receipt.receipt_id,
-        &prev_block_hash,
-        &block_hash,
-        salt,
-    )
+    create_hash_upgradable(protocol_version, &receipt.receipt_id, prev_block_hash, block_hash, salt)
 }
 
 /// Creates a new `data_id` from a given action hash, a block hash and a data index.
@@ -220,9 +263,9 @@ pub fn create_data_id(
 ) -> CryptoHash {
     create_hash_upgradable(
         protocol_version,
-        &action_hash,
-        &prev_block_hash,
-        &block_hash,
+        action_hash,
+        prev_block_hash,
+        block_hash,
         data_index as u64,
     )
 }
@@ -290,10 +333,8 @@ fn create_nonce_with_nonce(base: &CryptoHash, salt: u64) -> CryptoHash {
     hash(&nonce)
 }
 
-pub fn index_to_bytes(index: u64) -> Vec<u8> {
-    let mut bytes = vec![];
-    bytes.write_u64::<LittleEndian>(index).expect("writing to bytes failed");
-    bytes
+pub fn index_to_bytes(index: u64) -> [u8; 8] {
+    index.to_le_bytes()
 }
 
 /// A wrapper around Option<T> that provides native Display trait.
@@ -350,27 +391,6 @@ macro_rules! unwrap_or_return {
     };
 }
 
-/// Macro to either return value if the result is Some, or exit function.
-#[macro_export]
-macro_rules! unwrap_option_or_return {
-    ($obj: expr, $ret: expr) => {
-        match $obj {
-            Some(value) => value,
-            None => {
-                return $ret;
-            }
-        }
-    };
-    ($obj: expr) => {
-        match $obj {
-            Some(value) => value,
-            None => {
-                return;
-            }
-        }
-    };
-}
-
 /// Converts timestamp in ns into DateTime UTC time.
 pub fn from_timestamp(timestamp: u64) -> DateTime<chrono::Utc> {
     DateTime::from_utc(
@@ -430,7 +450,7 @@ impl fmt::Debug for dyn CompiledContractCache {
 /// objects using tracing.
 ///
 /// tracing::debug!(target: "diagnostic", value=%ser(&object));
-pub fn ser<'a, T>(object: &'a T) -> Serializable<'a, T>
+pub fn ser<T>(object: &T) -> Serializable<'_, T>
 where
     T: serde::Serialize,
 {
