@@ -3,6 +3,7 @@ use serde::{Deserialize, Serialize};
 use crate::types::Balance;
 
 /// Data structure for semver version and github tag or commit.
+#[cfg_attr(feature = "deepsize_feature", derive(deepsize::DeepSizeOf))]
 #[derive(Serialize, Deserialize, Clone, Debug, Default)]
 pub struct Version {
     pub version: String,
@@ -13,14 +14,11 @@ pub struct Version {
 pub type DbVersion = u32;
 
 /// Current version of the database.
-pub const DB_VERSION: DbVersion = 29;
+pub const DB_VERSION: DbVersion = 31;
 
 /// Protocol version type.
 pub use near_primitives_core::types::ProtocolVersion;
 
-/// Oldest supported version by this client.
-pub const OLDEST_BACKWARD_COMPATIBLE_PROTOCOL_VERSION: ProtocolVersion = 34;
-
 /// Minimum gas price proposed in NEP 92 and the associated protocol version
 pub const MIN_GAS_PRICE_NEP_92: Balance = 1_000_000_000;
 pub const MIN_PROTOCOL_VERSION_NEP_92: ProtocolVersion = 31;
@@ -83,6 +81,8 @@ pub enum ProtocolFeature {
     // stable features
     ForwardChunkParts,
     RectifyInflation,
+    /// Add `AccessKey` nonce range by setting nonce to `(block_height - 1) * 1e6`, see
+    /// <https://github.com/near/nearcore/issues/3779>.
     AccessKeyNonceRange,
     FixApplyChunks,
     LowerStorageCost,
@@ -99,7 +99,7 @@ pub enum ProtocolFeature {
     MathExtension,
     /// Restore receipts that were previously stuck because of
     /// <https://github.com/near/nearcore/pull/4228>.
-    RestoreReceiptsAfterFix,
+    RestoreReceiptsAfterFixApplyChunks,
     /// This feature switch our WASM engine implementation from wasmer 0.* to
     /// wasmer 2.*, bringing better performance and reliability.
     ///
@@ -120,27 +120,49 @@ pub enum ProtocolFeature {
     /// Limit number of wasm functions in one contract. See
     /// <https://github.com/near/nearcore/pull/4954> for more details.
     LimitContractFunctionsNumber,
+    BlockHeaderV3,
+    /// Changes how we select validators for epoch and how we select validators within epoch. See
+    /// https://github.com/near/NEPs/pull/167 for general description, note that we would not
+    /// introduce chunk-only validators with this feature
+    AliasValidatorSelectionAlgorithm,
+    /// Make block producers produce chunks for the same block they would later produce to avoid
+    /// network delays
+    SynchronizeBlockChunkProduction,
+    /// Change the algorithm to count WASM stack usage to avoid undercounting in
+    /// some cases.
+    CorrectStackLimit,
+    /// Add `AccessKey` nonce range for implicit accounts, as in `AccessKeyNonceRange` feature.
+    AccessKeyNonceForImplicitAccounts,
 
     // nightly features
-    #[cfg(feature = "protocol_feature_block_header_v3")]
-    BlockHeaderV3,
     #[cfg(feature = "protocol_feature_alt_bn128")]
     AltBn128,
     #[cfg(feature = "protocol_feature_chunk_only_producers")]
     ChunkOnlyProducers,
     #[cfg(feature = "protocol_feature_routing_exchange_algorithm")]
     RoutingExchangeAlgorithm,
+    /// In case not all validator seats are occupied our algorithm provide incorrect minimal seat
+    /// price - it reports as alpha * sum_stake instead of alpha * sum_stake / (1 - alpha), where
+    /// alpha is min stake ratio
+    #[cfg(feature = "protocol_feature_fix_staking_threshold")]
+    FixStakingThreshold,
 }
 
-/// Current latest stable version of the protocol.
+/// Both, outgoing and incoming tcp connections to peers, will be rejected if `peer's`
+/// protocol version is lower than this.
+pub const PEER_MIN_ALLOWED_PROTOCOL_VERSION: ProtocolVersion = MAIN_NET_PROTOCOL_VERSION - 2;
+
+/// Current protocol version used on the main net.
 /// Some features (e. g. FixStorageUsage) require that there is at least one epoch with exactly
 /// the corresponding version
-#[cfg(not(feature = "nightly_protocol"))]
-pub const PROTOCOL_VERSION: ProtocolVersion = 49;
+const MAIN_NET_PROTOCOL_VERSION: ProtocolVersion = 51;
 
+/// Version used by this binary.
+#[cfg(not(feature = "nightly_protocol"))]
+pub const PROTOCOL_VERSION: ProtocolVersion = MAIN_NET_PROTOCOL_VERSION;
 /// Current latest nightly version of the protocol.
 #[cfg(feature = "nightly_protocol")]
-pub const PROTOCOL_VERSION: ProtocolVersion = 124;
+pub const PROTOCOL_VERSION: ProtocolVersion = 126;
 
 impl ProtocolFeature {
     pub const fn protocol_version(self) -> ProtocolVersion {
@@ -158,23 +180,28 @@ impl ProtocolFeature {
             | ProtocolFeature::CapMaxGasPrice
             | ProtocolFeature::CountRefundReceiptsInGasLimit
             | ProtocolFeature::MathExtension => 46,
-            ProtocolFeature::RestoreReceiptsAfterFix => 47,
+            ProtocolFeature::RestoreReceiptsAfterFixApplyChunks => 47,
             ProtocolFeature::Wasmer2
             | ProtocolFeature::LowerDataReceiptAndEcrecoverBaseCost
             | ProtocolFeature::LowerRegularOpCost
             | ProtocolFeature::SimpleNightshade => 48,
             ProtocolFeature::LowerRegularOpCost2
-            | ProtocolFeature::LimitContractFunctionsNumber => 49,
+            | ProtocolFeature::LimitContractFunctionsNumber
+            | ProtocolFeature::BlockHeaderV3
+            | ProtocolFeature::AliasValidatorSelectionAlgorithm => 49,
+            ProtocolFeature::SynchronizeBlockChunkProduction
+            | ProtocolFeature::CorrectStackLimit => 50,
+            ProtocolFeature::AccessKeyNonceForImplicitAccounts => 51,
 
             // Nightly features
             #[cfg(feature = "protocol_feature_alt_bn128")]
             ProtocolFeature::AltBn128 => 105,
-            #[cfg(feature = "protocol_feature_block_header_v3")]
-            ProtocolFeature::BlockHeaderV3 => 109,
             #[cfg(feature = "protocol_feature_chunk_only_producers")]
-            ProtocolFeature::ChunkOnlyProducers => 115,
+            ProtocolFeature::ChunkOnlyProducers => 124,
             #[cfg(feature = "protocol_feature_routing_exchange_algorithm")]
             ProtocolFeature::RoutingExchangeAlgorithm => 117,
+            #[cfg(feature = "protocol_feature_fix_staking_threshold")]
+            ProtocolFeature::FixStakingThreshold => 126,
         }
     }
 }
