@@ -15,11 +15,9 @@ use near_crypto::{PublicKey, Signature};
 use crate::account::{AccessKey, AccessKeyPermission, Account, FunctionCallPermission};
 use crate::block::{Block, BlockHeader};
 use crate::block_header::{
-    BlockHeaderInnerLite, BlockHeaderInnerRest, BlockHeaderInnerRestV2, BlockHeaderV1,
-    BlockHeaderV2,
+    BlockHeaderInnerLite, BlockHeaderInnerRest, BlockHeaderInnerRestV2, BlockHeaderInnerRestV3,
+    BlockHeaderV1, BlockHeaderV2, BlockHeaderV3,
 };
-#[cfg(feature = "protocol_feature_block_header_v3")]
-use crate::block_header::{BlockHeaderInnerRestV3, BlockHeaderV3};
 use crate::challenge::{Challenge, ChallengesResult};
 use crate::contract::ContractCode;
 use crate::errors::TxExecutionError;
@@ -32,11 +30,10 @@ use crate::serialize::{
     base64_format, from_base64, option_base64_format, option_u128_dec_format, to_base64,
     u128_dec_format, u64_dec_format,
 };
-#[cfg(not(feature = "protocol_feature_block_header_v3"))]
-use crate::sharding::ShardChunkHeaderV2;
-use crate::sharding::{ChunkHash, ShardChunk, ShardChunkHeader, ShardChunkHeaderInner};
-#[cfg(feature = "protocol_feature_block_header_v3")]
-use crate::sharding::{ShardChunkHeaderInnerV2, ShardChunkHeaderV3};
+use crate::sharding::{
+    ChunkHash, ShardChunk, ShardChunkHeader, ShardChunkHeaderInner, ShardChunkHeaderInnerV2,
+    ShardChunkHeaderV3,
+};
 use crate::transaction::{
     Action, AddKeyAction, CreateAccountAction, DeleteAccountAction, DeleteKeyAction,
     DeployContractAction, ExecutionMetadata, ExecutionOutcome, ExecutionOutcomeWithIdAndProof,
@@ -52,6 +49,7 @@ use crate::version::{ProtocolVersion, Version};
 use validator_stake_view::ValidatorStakeView;
 
 /// A view of the account
+#[cfg_attr(feature = "deepsize_feature", derive(deepsize::DeepSizeOf))]
 #[derive(BorshSerialize, BorshDeserialize, Serialize, Deserialize, Debug, Eq, PartialEq, Clone)]
 pub struct AccountView {
     #[serde(with = "u128_dec_format")]
@@ -66,6 +64,7 @@ pub struct AccountView {
 }
 
 /// A view of the contract code.
+#[cfg_attr(feature = "deepsize_feature", derive(deepsize::DeepSizeOf))]
 #[derive(BorshSerialize, BorshDeserialize, Serialize, Deserialize, PartialEq, Eq, Debug, Clone)]
 pub struct ContractCodeView {
     #[serde(rename = "code_base64", with = "base64_format")]
@@ -138,6 +137,7 @@ impl From<ContractCodeView> for ContractCode {
     }
 }
 
+#[cfg_attr(feature = "deepsize_feature", derive(deepsize::DeepSizeOf))]
 #[derive(BorshSerialize, BorshDeserialize, Serialize, Deserialize, Debug, Eq, PartialEq, Clone)]
 pub enum AccessKeyPermissionView {
     FunctionCall {
@@ -177,6 +177,7 @@ impl From<AccessKeyPermissionView> for AccessKeyPermission {
     }
 }
 
+#[cfg_attr(feature = "deepsize_feature", derive(deepsize::DeepSizeOf))]
 #[derive(BorshSerialize, BorshDeserialize, Serialize, Deserialize, Debug, Eq, PartialEq, Clone)]
 pub struct AccessKeyView {
     pub nonce: Nonce,
@@ -199,6 +200,7 @@ impl From<AccessKeyView> for AccessKey {
 pub type TrieProofPath = Vec<String>;
 
 /// Item of the state, key and value are serialized in base64 and proof for inclusion of given state item.
+#[cfg_attr(feature = "deepsize_feature", derive(deepsize::DeepSizeOf))]
 #[derive(BorshSerialize, BorshDeserialize, Serialize, Deserialize, Debug, PartialEq, Eq, Clone)]
 pub struct StateItem {
     pub key: String,
@@ -206,12 +208,14 @@ pub struct StateItem {
     pub proof: TrieProofPath,
 }
 
+#[cfg_attr(feature = "deepsize_feature", derive(deepsize::DeepSizeOf))]
 #[derive(BorshSerialize, BorshDeserialize, Serialize, Deserialize, Debug, PartialEq, Eq, Clone)]
 pub struct ViewStateResult {
     pub values: Vec<StateItem>,
     pub proof: TrieProofPath,
 }
 
+#[cfg_attr(feature = "deepsize_feature", derive(deepsize::DeepSizeOf))]
 #[derive(
     BorshSerialize, BorshDeserialize, Serialize, Deserialize, Debug, PartialEq, Eq, Clone, Default,
 )]
@@ -220,18 +224,21 @@ pub struct CallResult {
     pub logs: Vec<String>,
 }
 
+#[cfg_attr(feature = "deepsize_feature", derive(deepsize::DeepSizeOf))]
 #[derive(BorshSerialize, BorshDeserialize, Serialize, Deserialize, Debug, PartialEq, Eq, Clone)]
 pub struct QueryError {
     pub error: String,
     pub logs: Vec<String>,
 }
 
+#[cfg_attr(feature = "deepsize_feature", derive(deepsize::DeepSizeOf))]
 #[derive(BorshSerialize, BorshDeserialize, Serialize, Deserialize, Debug, PartialEq, Eq, Clone)]
 pub struct AccessKeyInfoView {
     pub public_key: PublicKey,
     pub access_key: AccessKeyView,
 }
 
+#[cfg_attr(feature = "deepsize_feature", derive(deepsize::DeepSizeOf))]
 #[derive(BorshSerialize, BorshDeserialize, Serialize, Deserialize, Debug, PartialEq, Eq, Clone)]
 pub struct AccessKeyList {
     pub keys: Vec<AccessKeyInfoView>,
@@ -243,6 +250,7 @@ impl FromIterator<AccessKeyInfoView> for AccessKeyList {
     }
 }
 
+#[cfg_attr(feature = "deepsize_feature", derive(deepsize::DeepSizeOf))]
 #[derive(BorshSerialize, BorshDeserialize, Debug, PartialEq, Eq, Clone)]
 pub enum QueryResponseKind {
     ViewAccount(AccountView),
@@ -253,6 +261,7 @@ pub enum QueryResponseKind {
     AccessKeyList(AccessKeyList),
 }
 
+#[cfg_attr(feature = "deepsize_feature", derive(deepsize::DeepSizeOf))]
 #[derive(BorshSerialize, BorshDeserialize, Serialize, Deserialize, Debug, PartialEq, Eq, Clone)]
 #[serde(tag = "request_type", rename_all = "snake_case")]
 pub enum QueryRequest {
@@ -282,6 +291,7 @@ pub enum QueryRequest {
     },
 }
 
+#[cfg_attr(feature = "deepsize_feature", derive(deepsize::DeepSizeOf))]
 #[derive(BorshSerialize, BorshDeserialize, Debug, PartialEq, Eq, Clone)]
 pub struct QueryResponse {
     pub kind: QueryResponseKind,
@@ -289,6 +299,7 @@ pub struct QueryResponse {
     pub block_hash: CryptoHash,
 }
 
+#[cfg_attr(feature = "deepsize_feature", derive(deepsize::DeepSizeOf))]
 #[derive(Serialize, Deserialize, Debug)]
 pub struct StatusSyncInfo {
     pub latest_block_hash: CryptoHash,
@@ -302,6 +313,7 @@ pub struct StatusSyncInfo {
 }
 
 // TODO: add more information to ValidatorInfo
+#[cfg_attr(feature = "deepsize_feature", derive(deepsize::DeepSizeOf))]
 #[derive(Serialize, Deserialize, Debug, PartialEq, Eq)]
 pub struct ValidatorInfo {
     pub account_id: AccountId,
@@ -309,6 +321,7 @@ pub struct ValidatorInfo {
 }
 
 // TODO: add more information to status.
+#[cfg_attr(feature = "deepsize_feature", derive(deepsize::DeepSizeOf))]
 #[derive(Serialize, Deserialize, Debug)]
 pub struct StatusResponse {
     /// Binary version.
@@ -344,7 +357,6 @@ impl From<Challenge> for ChallengeView {
 #[derive(Serialize, Deserialize, Debug, Clone)]
 pub struct BlockHeaderView {
     pub height: BlockHeight,
-    #[cfg(feature = "protocol_feature_block_header_v3")]
     pub prev_height: Option<BlockHeight>,
     pub epoch_id: CryptoHash,
     pub next_epoch_id: CryptoHash,
@@ -366,7 +378,6 @@ pub struct BlockHeaderView {
     pub chunk_mask: Vec<bool>,
     #[serde(with = "u128_dec_format")]
     pub gas_price: Balance,
-    #[cfg(feature = "protocol_feature_block_header_v3")]
     pub block_ordinal: Option<NumBlocks>,
     /// TODO(2271): deprecated.
     #[serde(with = "u128_dec_format")]
@@ -381,7 +392,6 @@ pub struct BlockHeaderView {
     pub last_ds_final_block: CryptoHash,
     pub next_bp_hash: CryptoHash,
     pub block_merkle_root: CryptoHash,
-    #[cfg(feature = "protocol_feature_block_header_v3")]
     pub epoch_sync_data_hash: Option<CryptoHash>,
     pub approvals: Vec<Option<Signature>>,
     pub signature: Signature,
@@ -392,25 +402,23 @@ impl From<BlockHeader> for BlockHeaderView {
     fn from(header: BlockHeader) -> Self {
         Self {
             height: header.height(),
-            #[cfg(feature = "protocol_feature_block_header_v3")]
             prev_height: header.prev_height(),
             epoch_id: header.epoch_id().0,
             next_epoch_id: header.next_epoch_id().0,
-            hash: header.hash().clone(),
-            prev_hash: header.prev_hash().clone(),
-            prev_state_root: header.prev_state_root().clone(),
-            chunk_receipts_root: header.chunk_receipts_root().clone(),
-            chunk_headers_root: header.chunk_headers_root().clone(),
-            chunk_tx_root: header.chunk_tx_root().clone(),
+            hash: *header.hash(),
+            prev_hash: *header.prev_hash(),
+            prev_state_root: *header.prev_state_root(),
+            chunk_receipts_root: *header.chunk_receipts_root(),
+            chunk_headers_root: *header.chunk_headers_root(),
+            chunk_tx_root: *header.chunk_tx_root(),
             chunks_included: header.chunks_included(),
-            challenges_root: header.challenges_root().clone(),
-            outcome_root: header.outcome_root().clone(),
+            challenges_root: *header.challenges_root(),
+            outcome_root: *header.outcome_root(),
             timestamp: header.raw_timestamp(),
             timestamp_nanosec: header.raw_timestamp(),
-            random_value: header.random_value().clone(),
+            random_value: *header.random_value(),
             validator_proposals: header.validator_proposals().map(Into::into).collect(),
             chunk_mask: header.chunk_mask().to_vec(),
-            #[cfg(feature = "protocol_feature_block_header_v3")]
             block_ordinal: if header.block_ordinal() != 0 {
                 Some(header.block_ordinal())
             } else {
@@ -421,11 +429,10 @@ impl From<BlockHeader> for BlockHeaderView {
             validator_reward: 0,
             total_supply: header.total_supply(),
             challenges_result: header.challenges_result().clone(),
-            last_final_block: header.last_final_block().clone(),
-            last_ds_final_block: header.last_ds_final_block().clone(),
-            next_bp_hash: header.next_bp_hash().clone(),
-            block_merkle_root: header.block_merkle_root().clone(),
-            #[cfg(feature = "protocol_feature_block_header_v3")]
+            last_final_block: *header.last_final_block(),
+            last_ds_final_block: *header.last_ds_final_block(),
+            next_bp_hash: *header.next_bp_hash(),
+            block_merkle_root: *header.block_merkle_root(),
             epoch_sync_data_hash: header.epoch_sync_data_hash(),
             approvals: header.approvals().to_vec(),
             signature: header.signature().clone(),
@@ -446,21 +453,14 @@ impl From<BlockHeaderView> for BlockHeader {
             next_bp_hash: view.next_bp_hash,
             block_merkle_root: view.block_merkle_root,
         };
-        #[cfg(not(feature = "protocol_feature_block_header_v3"))]
-        let last_header_v2_version: Option<u32> = None;
-        #[cfg(feature = "protocol_feature_block_header_v3")]
         let last_header_v2_version =
             Some(crate::version::ProtocolFeature::BlockHeaderV3.protocol_version() - 1);
         if view.latest_protocol_version <= 29 {
-            #[cfg(feature = "protocol_feature_block_header_v3")]
             let validator_proposals = view
                 .validator_proposals
                 .into_iter()
                 .map(|v| v.into_validator_stake().into_v1())
                 .collect();
-            #[cfg(not(feature = "protocol_feature_block_header_v3"))]
-            let validator_proposals =
-                view.validator_proposals.into_iter().map(Into::into).collect();
             let mut header = BlockHeaderV1 {
                 prev_hash: view.prev_hash,
                 inner_lite,
@@ -489,15 +489,11 @@ impl From<BlockHeaderView> for BlockHeader {
         } else if last_header_v2_version.is_none()
             || view.latest_protocol_version <= last_header_v2_version.unwrap()
         {
-            #[cfg(feature = "protocol_feature_block_header_v3")]
             let validator_proposals = view
                 .validator_proposals
                 .into_iter()
                 .map(|v| v.into_validator_stake().into_v1())
                 .collect();
-            #[cfg(not(feature = "protocol_feature_block_header_v3"))]
-            let validator_proposals =
-                view.validator_proposals.into_iter().map(Into::into).collect();
             let mut header = BlockHeaderV2 {
                 prev_hash: view.prev_hash,
                 inner_lite,
@@ -523,49 +519,42 @@ impl From<BlockHeaderView> for BlockHeader {
             header.init();
             BlockHeader::BlockHeaderV2(Box::new(header))
         } else {
-            #[cfg(not(feature = "protocol_feature_block_header_v3"))]
-            unreachable!();
-            #[cfg(feature = "protocol_feature_block_header_v3")]
-            {
-                let mut header = BlockHeaderV3 {
-                    prev_hash: view.prev_hash,
-                    inner_lite,
-                    inner_rest: BlockHeaderInnerRestV3 {
-                        chunk_receipts_root: view.chunk_receipts_root,
-                        chunk_headers_root: view.chunk_headers_root,
-                        chunk_tx_root: view.chunk_tx_root,
-                        challenges_root: view.challenges_root,
-                        random_value: view.random_value,
-                        validator_proposals: view
-                            .validator_proposals
-                            .into_iter()
-                            .map(Into::into)
-                            .collect(),
-                        chunk_mask: view.chunk_mask,
-                        gas_price: view.gas_price,
-                        block_ordinal: match view.block_ordinal {
-                            Some(value) => value,
-                            None => 0,
-                        },
-                        total_supply: view.total_supply,
-                        challenges_result: view.challenges_result,
-                        last_final_block: view.last_final_block,
-                        last_ds_final_block: view.last_ds_final_block,
-                        prev_height: view.prev_height.unwrap_or_default(),
-                        epoch_sync_data_hash: view.epoch_sync_data_hash,
-                        approvals: view.approvals.clone(),
-                        latest_protocol_version: view.latest_protocol_version,
-                    },
-                    signature: view.signature,
-                    hash: CryptoHash::default(),
-                };
-                header.init();
-                BlockHeader::BlockHeaderV3(Box::new(header))
-            }
+            let mut header = BlockHeaderV3 {
+                prev_hash: view.prev_hash,
+                inner_lite,
+                inner_rest: BlockHeaderInnerRestV3 {
+                    chunk_receipts_root: view.chunk_receipts_root,
+                    chunk_headers_root: view.chunk_headers_root,
+                    chunk_tx_root: view.chunk_tx_root,
+                    challenges_root: view.challenges_root,
+                    random_value: view.random_value,
+                    validator_proposals: view
+                        .validator_proposals
+                        .into_iter()
+                        .map(Into::into)
+                        .collect(),
+                    chunk_mask: view.chunk_mask,
+                    gas_price: view.gas_price,
+                    block_ordinal: view.block_ordinal.unwrap_or(0),
+                    total_supply: view.total_supply,
+                    challenges_result: view.challenges_result,
+                    last_final_block: view.last_final_block,
+                    last_ds_final_block: view.last_ds_final_block,
+                    prev_height: view.prev_height.unwrap_or_default(),
+                    epoch_sync_data_hash: view.epoch_sync_data_hash,
+                    approvals: view.approvals.clone(),
+                    latest_protocol_version: view.latest_protocol_version,
+                },
+                signature: view.signature,
+                hash: CryptoHash::default(),
+            };
+            header.init();
+            BlockHeader::BlockHeaderV3(Box::new(header))
         }
     }
 }
 
+#[cfg_attr(feature = "deepsize_feature", derive(deepsize::DeepSizeOf))]
 #[derive(Serialize, Deserialize, PartialEq, Eq, Debug, Clone, BorshDeserialize, BorshSerialize)]
 pub struct BlockHeaderInnerLiteView {
     pub height: BlockHeight,
@@ -606,7 +595,6 @@ impl From<BlockHeader> for BlockHeaderInnerLiteView {
                 next_bp_hash: header.inner_lite.next_bp_hash,
                 block_merkle_root: header.inner_lite.block_merkle_root,
             },
-            #[cfg(feature = "protocol_feature_block_header_v3")]
             BlockHeader::BlockHeaderV3(header) => BlockHeaderInnerLiteView {
                 height: header.inner_lite.height,
                 epoch_id: header.inner_lite.epoch_id.0,
@@ -693,7 +681,6 @@ impl From<ShardChunkHeader> for ChunkHeaderView {
     }
 }
 
-#[cfg(feature = "protocol_feature_block_header_v3")]
 impl From<ChunkHeaderView> for ShardChunkHeader {
     fn from(view: ChunkHeaderView) -> Self {
         let mut header = ShardChunkHeaderV3 {
@@ -721,34 +708,6 @@ impl From<ChunkHeaderView> for ShardChunkHeader {
     }
 }
 
-#[cfg(not(feature = "protocol_feature_block_header_v3"))]
-impl From<ChunkHeaderView> for ShardChunkHeader {
-    fn from(view: ChunkHeaderView) -> Self {
-        let mut header = ShardChunkHeaderV2 {
-            inner: ShardChunkHeaderInner {
-                prev_block_hash: view.prev_block_hash,
-                prev_state_root: view.prev_state_root,
-                outcome_root: view.outcome_root,
-                encoded_merkle_root: view.encoded_merkle_root,
-                encoded_length: view.encoded_length,
-                height_created: view.height_created,
-                shard_id: view.shard_id,
-                gas_used: view.gas_used,
-                gas_limit: view.gas_limit,
-                balance_burnt: view.balance_burnt,
-                outgoing_receipts_root: view.outgoing_receipts_root,
-                tx_root: view.tx_root,
-                validator_proposals: view.validator_proposals.into_iter().map(Into::into).collect(),
-            },
-            height_included: view.height_included,
-            signature: view.signature,
-            hash: ChunkHash::default(),
-        };
-        header.init();
-        ShardChunkHeader::V2(header)
-    }
-}
-
 #[derive(Serialize, Deserialize, Debug)]
 pub struct BlockView {
     pub author: AccountId,
@@ -793,6 +752,7 @@ impl ChunkView {
     }
 }
 
+#[cfg_attr(feature = "deepsize_feature", derive(deepsize::DeepSizeOf))]
 #[derive(Serialize, Deserialize, Clone, Debug, BorshSerialize, BorshDeserialize, PartialEq, Eq)]
 pub enum ActionView {
     CreateAccount,
@@ -904,6 +864,7 @@ impl TryFrom<ActionView> for Action {
     }
 }
 
+#[cfg_attr(feature = "deepsize_feature", derive(deepsize::DeepSizeOf))]
 #[derive(Serialize, Deserialize, Debug, BorshSerialize, BorshDeserialize, PartialEq, Eq, Clone)]
 pub struct SignedTransactionView {
     pub signer_id: AccountId,
@@ -935,6 +896,7 @@ impl From<SignedTransaction> for SignedTransactionView {
     }
 }
 
+#[cfg_attr(feature = "deepsize_feature", derive(deepsize::DeepSizeOf))]
 #[derive(BorshSerialize, BorshDeserialize, Serialize, Deserialize, PartialEq, Eq, Clone)]
 pub enum FinalExecutionStatus {
     /// The execution has not yet started.
@@ -955,7 +917,7 @@ impl fmt::Debug for FinalExecutionStatus {
             FinalExecutionStatus::Failure(e) => f.write_fmt(format_args!("Failure({:?})", e)),
             FinalExecutionStatus::SuccessValue(v) => f.write_fmt(format_args!(
                 "SuccessValue({})",
-                logging::pretty_utf8(&from_base64(&v).unwrap())
+                logging::pretty_utf8(&from_base64(v).unwrap())
             )),
         }
     }
@@ -967,6 +929,7 @@ impl Default for FinalExecutionStatus {
     }
 }
 
+#[cfg_attr(feature = "deepsize_feature", derive(deepsize::DeepSizeOf))]
 #[derive(BorshSerialize, BorshDeserialize, Serialize, Deserialize, Debug, PartialEq, Eq, Clone)]
 pub enum ServerError {
     TxExecutionError(TxExecutionError),
@@ -974,6 +937,7 @@ pub enum ServerError {
     Closed,
 }
 
+#[cfg_attr(feature = "deepsize_feature", derive(deepsize::DeepSizeOf))]
 #[derive(BorshSerialize, BorshDeserialize, Serialize, Deserialize, PartialEq, Eq, Clone)]
 pub enum ExecutionStatusView {
     /// The execution is pending or unknown.
@@ -994,7 +958,7 @@ impl fmt::Debug for ExecutionStatusView {
             ExecutionStatusView::Failure(e) => f.write_fmt(format_args!("Failure({:?})", e)),
             ExecutionStatusView::SuccessValue(v) => f.write_fmt(format_args!(
                 "SuccessValue({})",
-                logging::pretty_utf8(&from_base64(&v).unwrap())
+                logging::pretty_utf8(&from_base64(v).unwrap())
             )),
             ExecutionStatusView::SuccessReceiptId(receipt_id) => {
                 f.write_fmt(format_args!("SuccessReceiptId({})", receipt_id))
@@ -1016,6 +980,7 @@ impl From<ExecutionStatus> for ExecutionStatusView {
     }
 }
 
+#[cfg_attr(feature = "deepsize_feature", derive(deepsize::DeepSizeOf))]
 #[derive(BorshSerialize, BorshDeserialize, Serialize, Deserialize, PartialEq, Clone, Eq, Debug)]
 pub struct CostGasUsed {
     pub cost_category: String,
@@ -1024,10 +989,11 @@ pub struct CostGasUsed {
     pub gas_used: Gas,
 }
 
+#[cfg_attr(feature = "deepsize_feature", derive(deepsize::DeepSizeOf))]
 #[derive(BorshSerialize, BorshDeserialize, Serialize, Deserialize, PartialEq, Clone, Eq, Debug)]
 pub struct ExecutionMetadataView {
-    version: u32,
-    gas_profile: Option<Vec<CostGasUsed>>,
+    pub version: u32,
+    pub gas_profile: Option<Vec<CostGasUsed>>,
 }
 
 impl Default for ExecutionMetadataView {
@@ -1040,32 +1006,48 @@ impl From<ExecutionMetadata> for ExecutionMetadataView {
     fn from(metadata: ExecutionMetadata) -> Self {
         let gas_profile = match metadata {
             ExecutionMetadata::V1 => None,
-            ExecutionMetadata::V2(profile_data) => Some(
-                Cost::ALL
+            ExecutionMetadata::V2(profile_data) => {
+                let mut costs: Vec<_> = Cost::ALL
                     .iter()
                     .filter(|&cost| profile_data[*cost] > 0)
                     .map(|&cost| CostGasUsed {
                         cost_category: match cost {
                             Cost::ActionCost { .. } => "ACTION_COST",
                             Cost::ExtCost { .. } => "WASM_HOST_COST",
+                            Cost::WasmInstruction => "WASM_HOST_COST",
                         }
                         .to_string(),
                         cost: match cost {
                             Cost::ActionCost { action_cost_kind: action_cost } => {
-                                format!("{:?}", action_cost)
+                                format!("{:?}", action_cost).to_ascii_uppercase()
                             }
-                            Cost::ExtCost { ext_cost_kind: ext_cost } => format!("{:?}", ext_cost),
-                        }
-                        .to_ascii_uppercase(),
+                            Cost::ExtCost { ext_cost_kind: ext_cost } => {
+                                format!("{:?}", ext_cost).to_ascii_uppercase()
+                            }
+                            Cost::WasmInstruction => "WASM_INSTRUCTION".to_string(),
+                        },
                         gas_used: profile_data[cost],
                     })
-                    .collect(),
-            ),
+                    .collect();
+
+                // The order doesn't really matter, but the default one is just
+                // historical, which is especially unintuitive, so let's sort
+                // lexicographically.
+                //
+                // Can't `sort_by_key` here because lifetime inference in
+                // closures is limited.
+                costs.sort_by(|lhs, rhs| {
+                    lhs.cost_category.cmp(&rhs.cost_category).then(lhs.cost.cmp(&rhs.cost))
+                });
+
+                Some(costs)
+            }
         };
         ExecutionMetadataView { version: 1, gas_profile }
     }
 }
 
+#[cfg_attr(feature = "deepsize_feature", derive(deepsize::DeepSizeOf))]
 #[derive(BorshSerialize, BorshDeserialize, Serialize, Deserialize, Debug, Clone, PartialEq, Eq)]
 pub struct ExecutionOutcomeView {
     /// Logs from this transaction or receipt.
@@ -1103,6 +1085,7 @@ impl From<ExecutionOutcome> for ExecutionOutcomeView {
     }
 }
 
+#[cfg_attr(feature = "deepsize_feature", derive(deepsize::DeepSizeOf))]
 #[derive(BorshSerialize, BorshDeserialize, Serialize, Deserialize, Debug, PartialEq, Eq, Clone)]
 pub struct ExecutionOutcomeWithIdView {
     pub proof: MerklePath,
@@ -1130,6 +1113,7 @@ pub enum FinalExecutionOutcomeViewEnum {
 }
 
 /// Final execution outcome of the transaction and all of subsequent the receipts.
+#[cfg_attr(feature = "deepsize_feature", derive(deepsize::DeepSizeOf))]
 #[derive(BorshSerialize, BorshDeserialize, Serialize, Deserialize, PartialEq, Eq, Clone)]
 pub struct FinalExecutionOutcomeView {
     /// Execution status. Contains the result in case of successful execution.
@@ -1158,6 +1142,7 @@ impl fmt::Debug for FinalExecutionOutcomeView {
 
 /// Final execution outcome of the transaction and all of subsequent the receipts. Also includes
 /// the generated receipt.
+#[cfg_attr(feature = "deepsize_feature", derive(deepsize::DeepSizeOf))]
 #[derive(BorshSerialize, BorshDeserialize, Serialize, Deserialize, PartialEq, Eq, Clone, Debug)]
 pub struct FinalExecutionOutcomeWithReceiptView {
     /// Final outcome view without receipts
@@ -1173,7 +1158,6 @@ impl From<FinalExecutionOutcomeWithReceiptView> for FinalExecutionOutcomeView {
     }
 }
 
-#[cfg(feature = "protocol_feature_block_header_v3")]
 pub mod validator_stake_view {
     use crate::types::validator_stake::ValidatorStake;
     use borsh::{BorshDeserialize, BorshSerialize};
@@ -1189,6 +1173,7 @@ pub mod validator_stake_view {
 
     pub use super::ValidatorStakeViewV1;
 
+    #[cfg_attr(feature = "deepsize_feature", derive(deepsize::DeepSizeOf))]
     #[derive(
         BorshSerialize, BorshDeserialize, Serialize, Deserialize, Debug, Clone, Eq, PartialEq,
     )]
@@ -1224,6 +1209,7 @@ pub mod validator_stake_view {
     }
 
     #[cfg(feature = "protocol_feature_chunk_only_producers")]
+    #[cfg_attr(feature = "deepsize_feature", derive(deepsize::DeepSizeOf))]
     #[derive(
         BorshSerialize, BorshDeserialize, Serialize, Deserialize, Debug, Clone, Eq, PartialEq,
     )]
@@ -1267,39 +1253,7 @@ pub mod validator_stake_view {
     }
 }
 
-#[cfg(not(feature = "protocol_feature_block_header_v3"))]
-pub mod validator_stake_view {
-    use crate::types::validator_stake::ValidatorStake;
-    use near_primitives_core::types::AccountId;
-
-    pub use super::ValidatorStakeViewV1;
-    pub type ValidatorStakeView = ValidatorStakeViewV1;
-
-    impl ValidatorStakeView {
-        #[inline]
-        pub fn take_account_id(self) -> AccountId {
-            self.account_id
-        }
-
-        #[inline]
-        pub fn account_id(&self) -> &AccountId {
-            &self.account_id
-        }
-    }
-
-    impl From<ValidatorStake> for ValidatorStakeView {
-        fn from(stake: ValidatorStake) -> Self {
-            Self { account_id: stake.account_id, public_key: stake.public_key, stake: stake.stake }
-        }
-    }
-
-    impl From<ValidatorStakeView> for ValidatorStake {
-        fn from(view: ValidatorStakeView) -> Self {
-            Self { account_id: view.account_id, public_key: view.public_key, stake: view.stake }
-        }
-    }
-}
-
+#[cfg_attr(feature = "deepsize_feature", derive(deepsize::DeepSizeOf))]
 #[derive(BorshSerialize, BorshDeserialize, Serialize, Deserialize, Debug, Clone, Eq, PartialEq)]
 pub struct ValidatorStakeViewV1 {
     pub account_id: AccountId,
@@ -1308,6 +1262,7 @@ pub struct ValidatorStakeViewV1 {
     pub stake: Balance,
 }
 
+#[cfg_attr(feature = "deepsize_feature", derive(deepsize::DeepSizeOf))]
 #[derive(BorshSerialize, BorshDeserialize, Serialize, Deserialize, Clone, Debug, PartialEq, Eq)]
 pub struct ReceiptView {
     pub predecessor_id: AccountId,
@@ -1317,12 +1272,14 @@ pub struct ReceiptView {
     pub receipt: ReceiptEnumView,
 }
 
+#[cfg_attr(feature = "deepsize_feature", derive(deepsize::DeepSizeOf))]
 #[derive(BorshSerialize, BorshDeserialize, Serialize, Deserialize, Clone, Debug, PartialEq, Eq)]
 pub struct DataReceiverView {
     pub data_id: CryptoHash,
     pub receiver_id: AccountId,
 }
 
+#[cfg_attr(feature = "deepsize_feature", derive(deepsize::DeepSizeOf))]
 #[derive(BorshSerialize, BorshDeserialize, Serialize, Deserialize, Clone, Debug, PartialEq, Eq)]
 pub enum ReceiptEnumView {
     Action {
@@ -1417,7 +1374,8 @@ impl TryFrom<ReceiptView> for Receipt {
 }
 
 /// Information about this epoch validators and next epoch validators
-#[derive(BorshSerialize, BorshDeserialize, Serialize, Deserialize, Debug, PartialEq, Eq, Clone)]
+#[cfg_attr(feature = "deepsize_feature", derive(deepsize::DeepSizeOf))]
+#[derive(Serialize, Deserialize, Debug, PartialEq, Eq, Clone)]
 pub struct EpochValidatorInfo {
     /// Validators for the current epoch
     pub current_validators: Vec<CurrentEpochValidatorInfo>,
@@ -1437,13 +1395,15 @@ pub struct EpochValidatorInfo {
     pub epoch_height: EpochHeight,
 }
 
+#[cfg_attr(feature = "deepsize_feature", derive(deepsize::DeepSizeOf))]
 #[derive(BorshSerialize, BorshDeserialize, Serialize, Deserialize, Debug, PartialEq, Eq, Clone)]
 pub struct ValidatorKickoutView {
     pub account_id: AccountId,
     pub reason: ValidatorKickoutReason,
 }
 
-#[derive(BorshSerialize, BorshDeserialize, Serialize, Deserialize, Debug, PartialEq, Eq, Clone)]
+#[cfg_attr(feature = "deepsize_feature", derive(deepsize::DeepSizeOf))]
+#[derive(Serialize, Deserialize, Debug, PartialEq, Eq, Clone)]
 pub struct CurrentEpochValidatorInfo {
     pub account_id: AccountId,
     pub public_key: PublicKey,
@@ -1453,8 +1413,13 @@ pub struct CurrentEpochValidatorInfo {
     pub shards: Vec<ShardId>,
     pub num_produced_blocks: NumBlocks,
     pub num_expected_blocks: NumBlocks,
+    #[serde(default)]
+    pub num_produced_chunks: NumBlocks,
+    #[serde(default)]
+    pub num_expected_chunks: NumBlocks,
 }
 
+#[cfg_attr(feature = "deepsize_feature", derive(deepsize::DeepSizeOf))]
 #[derive(BorshSerialize, BorshDeserialize, Serialize, Deserialize, Debug, PartialEq, Eq, Clone)]
 pub struct NextEpochValidatorInfo {
     pub account_id: AccountId,
@@ -1464,6 +1429,7 @@ pub struct NextEpochValidatorInfo {
     pub shards: Vec<ShardId>,
 }
 
+#[cfg_attr(feature = "deepsize_feature", derive(deepsize::DeepSizeOf))]
 #[derive(Serialize, Deserialize, PartialEq, Eq, Debug, Clone, BorshDeserialize, BorshSerialize)]
 pub struct LightClientBlockView {
     pub prev_block_hash: CryptoHash,
@@ -1484,7 +1450,7 @@ pub struct LightClientBlockLiteView {
 impl From<BlockHeader> for LightClientBlockLiteView {
     fn from(header: BlockHeader) -> Self {
         Self {
-            prev_block_hash: header.prev_hash().clone(),
+            prev_block_hash: *header.prev_hash(),
             inner_rest_hash: hash(&header.inner_rest_bytes()),
             inner_lite: header.into(),
         }
